ifndef aiDumbBiped_wdl;
define aiDumbBiped_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
//
// title: Dumb AI Bipeds (01)
// class: AI
// type: USER
// image: aiDumbBiped01.pcx
// help:	Simple AI controlled bipeds.
// help: Used mostly for testing and "background" AIs.
//
// needs: gid01.wdl
// needs: bipedPhy01.wdl, bipedAnim01.wdl, bipedSnd01.wdl
//
// external:
//
//
//
// prefix: aiDumbBiped01_
// idcode: 01E
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// Debug Output Section

// enable: Display AI health?
// id: 1
// define	aiDumbBiped01_show_health;


// String used for debug output
string 	aiDumbBiped01_str[10];


// forces used to control AI
var	aiDumbBiped01_force_vec[3];   	// x,y,z forces
var	aiDumbBiped01_ang_force_vec[3];	// pan,tile,roll forces

///////////////////////////////////////////////////////////////////////
// Entity skill & flag definitions
define	_scale_range__01E,SKILL18; // scale range for dumb AI
define	_skin_range__01E,SKILL19;  // skin range for dumb AI
define	_life_span__01E,SKILL20;	// time left before removal




//-----------------------------------------------------------------------
// PathFollow Section

// path skills
define _path_node__01E, SKILL53;        // cur target node
define _path_vec__01E, SKILL54;         // cur target pos
define _path_vec_x__01E, SKILL54;
define _path_vec_y__01E, SKILL55;
define _path_vec_z__01E, SKILL56;

var	aiDumbBiped01_path_skills[6];	// used to store the skills of the current path


// Path node skills are used as follows:
//  0: behavior
//		0 - move along next path (stored in param1), if param1 == 0, use path 1.
//		1 - wait for the amount of time in param2, then behave like behavior 0 above
//	 1-4: parameters for behaviors
//	 5: Entity handle (or 0 if nobody is using this node).



////////////////////////////////////////////////////////////////////////


// Desc: move along next path (stored in param2), if param2 == 0, use path 1.
//
// uses: temp                      -> undef
//			aiDumbBiped01_path_skills -> set to the skills of current node
// 		my._path_node__029        -> set to id of current node
//			my._path_vec__029         -> undef
//
// mods: temp                      -> undef
//			aiDumbBiped01_path_skills -> changed to skills of next node
// 		my._path_node__029        -> chaged to id of next node
//			my._path_vec__029         -> changed to position(x,y,z) of next node
//
// return 0 if end of the path
//			 1 if legal path found
//			 2 if next node is "in use"
function	AIDumbBiped01_PathNext()
{

	// select which path to use
	if(aiDumbBiped01_path_skills[1] == 0)
	{
		my._path_node__01E = path_nextnode(my,my._path_node__01E,1);
	}
	else
	{
		// use the path stored in the param1
		my._path_node__01E = path_nextnode(my,my._path_node__01E,aiDumbBiped01_path_skills[1]);
	}


	if(my._path_node__01E != 0)
	{
		// set target vector to target node
		path_getnode(my,my._path_node__01E,my._path_vec__01E,aiDumbBiped01_path_skills);

		// if somebody is using this node, return "in use" value
		if(aiDumbBiped01_path_skills[5] != 0)
		{
			return(2); // next node in use
		}

		// set path as "in use" by this entity
		aiDumbBiped01_path_skills[5] = handle(my);
		path_setnode(my,my._path_node__01E,my._path_vec__01E,aiDumbBiped01_path_skills);
		return(1);
	}

	return(0);  // end of path
}


// Desc: action to preform on reaching a node
//
// return: 1 if we should continue pathfollowing
//			  0 if we should stop pathfollowing
function	AIDumbBiped01_PathOnNode()
{
	var	old_node;
	old_node = my._path_node__01E;

	// get skills for the current node
	path_getnode(my,old_node,temp,aiDumbBiped01_path_skills);

	// switch on node_skills[0]
	if(aiDumbBiped01_path_skills[0] == 1)
	{
		my._path_node__01E = 0;	// not following path for a while
		sleep(aiDumbBiped01_path_skills[2]);	// time out
		my._path_node__01E = old_node;	// continue follow

		// go to next node following path in node_skills[1]
		temp = AIDumbBiped01_PathNext();
	}
	else  // default action (node_skills[0] == 0)
	{
		// go to next node following path in node_skills[1]
		temp = AIDumbBiped01_PathNext();
	}

	// handle temp value
	if(temp == 2)	// next node is in use
	{
//		breakpoint;
		temp=temp;
	}

	// clear this node for use...
	path_getnode(my,old_node,temp,aiDumbBiped01_path_skills);
	aiDumbBiped01_path_skills[5] = 0;
	path_setnode(my,old_node,temp,aiDumbBiped01_path_skills);

	return(1);	// continue following node
}


//----------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////
// Desc: pan to face a vector
function	AIDumbBiped01_PanTo(panTo)
{
	panTo = ang(panTo-my.pan);

	// turn to face target angle
	if(panTo > my._max_ang_force_pan__003*TIME)
	{
		// turn anti-clockwise
		vec_set(aiDumbBiped01_ang_force_vec.pan,vector(my._max_ang_force_pan__003*TIME,0,0));
	}
	else
	{
		if(panTo < -my._max_ang_force_pan__003*TIME)
		{
			// turn anti-clockwise
			vec_set(aiDumbBiped01_ang_force_vec.pan,vector(-my._max_ang_force_pan__003*TIME,0,0));
		}
		else
		{
			// turn "exactly" to
			vec_set(aiDumbBiped01_ang_force_vec.pan,vector(panTo,0,0));
		}


	}
}




// Desc: update mode (call once per frame while active)
function	AIDumbBiped01_Walk_Update()
{
	while(1)
	{
		// Scan world

		//++ Get input

		// Path Follower?
		if(my._path_node__01E != 0)
		{
  			// we are a path follower
			vec_set(gid01_vec,my._path_vec__01E);
			vec_sub(gid01_vec,my.x);
			vec_to_angle(temp,gid01_vec);
			gid01_vec.z = 0;	// ignore the z value
			if(vec_length(gid01_vec) < 25)
			{
 				AIDumbBiped01_PathOnNode();	// preform node action
			}
			AIDumbBiped01_PanTo(temp.pan);
			vec_set(aiDumbBiped01_force_vec,vector(my._max_force_x__003,0,0));
		}
		else
		{
			// scale rotation and linear input by the force modifiers
			aiDumbBiped01_ang_force_vec.pan  = 0;
			aiDumbBiped01_ang_force_vec.tilt = 0;
			aiDumbBiped01_ang_force_vec.roll = 0;
			aiDumbBiped01_force_vec.x = my._max_force_x__003;
			aiDumbBiped01_force_vec.y = 0;
			aiDumbBiped01_force_vec.z = 0;
		}

		// Send forces/flags to biped01 update (which will handle physics, animation, and sound)
		BipedPhy01_Update(aiDumbBiped01_force_vec, aiDumbBiped01_ang_force_vec.pan);

		// Handle drowning damage
		if(my._biped01_state_mod == biped01_state_mod_drown_const)
		{
			// handle any damage from drowning (time based)
			my._health__003 -= bipedPhy01_drowning_damage_const*TIME;
		}

		// Handle Death
		if(my._health__003 <= 0)
		{
			// fade and remove
			my.transparent = on;
			my.alpha = 100;
			while(my.alpha > 0)
			{
				my.alpha -= 3*time;
				wait(1);
			}

			proc_kill(1);	// stop the other functions spawned by this entity
			wait(1);       // ...let them finish their run
			ent_remove(me);
			return;
		}



ifdef aiDumbBiped01_show_health;
	var	temp_vec[3];
	vec_set(temp_vec,my.x);
	if(vec_to_screen(temp_vec,camera))
	{
		str_for_num(aiDumbBiped01_str,my._health__003);
		draw_text(aiDumbBiped01_str,temp_vec.x,temp_vec.y,vector(100,100,255));
	}
endif;
		wait(1);
	}


}

// Desc: handle events
function AIDumbBiped01_Event()
{
	if((EVENT_TYPE == event_shoot) && (gid01_event_type == gid01_event_projectile_damage))
	{
		my._health__003 -= gid01_event_value;
	}

}

// Desc: Format the animation values (using defaults as neccessary).
//
// Assume: my is set to a AIDumbBiped01 entity
function	AIDumbBiped01_Set_Animation()
{
	if(my._walkswim_dist__002 == 0)   // walk distance . swim distance
	{
		my._walkswim_dist__002 = 4.04;
	}
	if(my._runcrawl_dist__002 == 0)   // run distance . crawl distance
	{
		my._runcrawl_dist__002 = 6.03;
	}
	if(my._standjump_time__002 == 0)   // stand time . jump time
	{
		my._standjump_time__002 = 4.02;
	}
	if(my._attackduck_time__002 == 0)   // attack time . duck time
	{
		my._attackduck_time__002 = 4.02;
	}
	if(my._deathdamage_time__002 == 0)   // death time . damage time
	{
		my._deathdamage_time__002 = 12.02;
	}
	if(my._runthreshold__002 == 0)	// run threshold
	{
		my._runthreshold__002 = 12;
	}
}

// Desc: Countdown timer, fade then remove the entity when it reaches zero
function	AIDumbBiped01_Life_Span()
{
	// wait
	sleep(my._life_span__01E);

	// fade and remove
	my.transparent = on;
	my.alpha = 100;
	while(my.alpha > 0)
	{
		my.alpha -= 3*time;
		wait(1);
	}

	proc_kill(1);	// stop the other functions spawned by this entity
	wait(1);       // ...let them finish their run
	ent_remove(me);
}



// action: AIDumbBiped01
// title: Dumb AI
// Desc: Simple walking AI.
// Desc: Limited obsticle avoidence. Can be killed.
//
//	Note: Good for "crowd scenes"
//
// uses _max_force_x__003, _max_force_y__003
// uses _max_ang_force_pan__003, _force_mult_003
// uses _move_min_z__003, _jump_height__003, _duck_height__003
// uses _walkswim_dist__002, _runcrawl_dist__002, _standjump_time__002
// uses _attackduck_time__002, _deathdamage_time__002, _runthreshold__002
// uses _health__003, _armor__003, _no_fall_dam__003
// uses _scale_range__01E, _skin_range__01E, _life_span__01E
//
//------------------
// section: Heath & Armor
// cntl: spin 1 x 5
// help: Reduced by damage. When <=0, player dies.
//	skill1: Health 100
//
// cntl: spin 0 x 5
// help: Reduces the damage that would otherwise be taken from health.
//	skill2: Armor 0
//
//------------------
// section: Movement
// help: Forward force used to calculate player speed.
// cntl: spin 0 x 0.25
// skill3: ForceX 7.5
//
// help: Force used to calculate player "side-step" speed.
// cntl: spin 0 x 0.25
// skill4: ForceY 3.75
//
// help: Force used to calculate the player's rotation speed.
// cntl: spin 0 x 0.25
// skill6: Pan 10
//
// help: Force multiplier when run/walk key is pressed:
// help:   > 1, player goes faster
// help:   < 1, player goes slower
// cntl: spin 0 x 0.25
// skill8: ForceMult 0.5
//
//------------------
// section: Limits
//
// help: Used to adjust the amount of upward sliding the AI can do.
// help: 0-can slide up onto anything under half its height, 0.99 no upward sliding.
// cntl: spin 0 0.99 0.05
// skill9: MinMoveZ 0.25
//
// help: Height that the player tries to jump to.
// cntl: spin 0 x 1
// skill10: Jump 75
//
// help: If set, player takes no falling damage.
// flag1: NoFallDamage 0
//
//
//------------------
// section: Animation
// help: Walk.Swim animation cycle distance.
// cntl: spin 0 x 1
// skill12: WalkSwimDist 4.04
//
// help: Run.Crawl animation cycle distance.
// cntl: spin 0 x 1
// skill13: RunCrawlDist 6.03
//
// help: Time in Stand.Jump animation cycle.
// cntl: spin 0 x 1
// skill14: StandJumpTime 4.02
//
// help: Time in Attack.Duck animation cycle.
// cntl: spin 0 x 1
// skill15: AttackDuckTime 4.02
//
// help: Time in Death.Damage animation cycle.
// cntl: spin 0 x 1
// skill16: DeathDamageTime 12.02
//
// help: If player is moving faster then this many quants,
// help:use running animation, otherwise use walking.
// cntl: spin 1 x 1
// skill17: RunThreshold 12
//
//--------------------------------
// section: Misc.
//	help:	The range of scaling that will be randomly applied on creation.
// skill18: RandScale 0
//
// help:	Skin Range: range of skins that the model can randomly select from
//	skill19: RandSkin  1
//
// help: The number of seconds this AI will exist (0 = unlimited)
//	skill20: LifeSpan  0
action	AIDumbBiped01
{
ifdef	GID01_OLD_COLLIDE;	// use the old collision code
	my.fat = OFF; my.narrow = ON;
ifelse;	// use A6 collison
	my.fat = ON; my.narrow = ON;
	c_setminmax(my);
endif;
	// set up biped specific variables

	my._gid01_id = gid01_ai_const;	// set ID to generic AI type

	// set up event handler
	my.event = AIDumbBiped01_Event;	// handle events
	my.enable_shoot = on;  	// gun shoot
	my.enable_scan = on;		// explosions


	// randomize appearence.
	if(my._scale_range__01E > 0)
	{
		temp = my._scale_range__01E/2;
		my.scale_x += random(my._scale_range__01E)-temp;
		my.scale_y += random(my._scale_range__01E)-temp;
		my.scale_z += random(my._scale_range__01E)-temp;
	}
	// select a random skin
	if(my._skin_range__01E > 1)
	{
		my.skin = 1 + random(my._skin_range__01E);
	}


	// default biped values
	if(my._health__003 == 0) { my._health__003 = 25; }
	//if(my._armor__003 == 0) { my._health__003 = 0; }
	if(my._max_force_x__003 == 0) { my._max_force_x__003 = 1.25; }
	if(my._max_force_y__003 == 0) { my._max_force_y__003 = 0.75; }
	if(my._max_ang_force_pan__003 == 0) { my._max_ang_force_pan__003 = 2; }
	if(my._force_mult_003 == 0) { my._force_mult_003 = 2; }
	//if(my._jump_height__003 == 0) { my._jump_height__003 = 0; }
	//if(my._duck_height__003 == 0) { my._duck_height__003 = 0; }

	AIDumbBiped01_Set_Animation();	// set the animation values for this AI

	// Check for path
	if(path_getnode(my,1,my._path_vec__01E,aiDumbBiped01_path_skills) != 0)
	{
		my._path_node__01E = 1;		// start at first node
		while(aiDumbBiped01_path_skills[5] != 0) // someone is using this node
		{
			my._path_node__01E += 1; // goto next node
			if(path_getnode(my,my._path_node__01E,my._path_vec__01E,aiDumbBiped01_path_skills) == 0)
			{
				// no room in path!
				my._path_node__01E = 0;
				break;
			}
		}


		if(my._path_node__01E != 0) // we have a valid node in a path
		{
			// move to start
			//path_nodepos(my,my._path_node__01E,my.x);
			vec_set(my.x,my._path_vec__01E);

			// mark this node as occupied
			aiDumbBiped01_path_skills[5] = handle(my);

			// set this as the current waypoint
			path_setnode(my,my._path_node__01E,my._path_vec__01E,aiDumbBiped01_path_skills);
		}

		wait(1);	// we should wait at least one frame so other pathfollow entities can see that we own this node during setup
	}

	// Start the animation update loop (runs until biped is removed)
	BipedAnim01_Update_Loop();
	// Start the sound loop (runs until biped is removed)
	BipedSnd01_Update_Loop();
	// Call the update loop
	AIDumbBiped01_Walk_Update();
	//
	if(my._life_span__01E > 0)
	{
		AIDumbBiped01_Life_Span();
	}

}


endif;