ifndef aiFixed_wdl;
define aiFixed_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Fixed place AI (01)
// class: AI
// type: USER
// image: aiFixed01.pcx
//help: AI for non-moving entities (gun turrets, search lights, etc.).
//
// needs: gid01.wdl, trigger00.wdl, particle00.wdl, weaponfx00.wdl, weapons00.wdl
// needs: plSelect.wdl
//
// external:
//
// prefix: aiFixed01_
// idcode: 02F
//
// ENDHEADER
// ---------------------------------------------------------------------



// WEAPON SKILLS
define	_weapon_index__02F,skill1;  	// Select the type of weapon this turret uses
define	_weapon_ammo__02F,skill2;  	// Number of shots (-1 for unlimited)
define	_weapon_reload__02F,skill3;  	// Number of seconds to reload the weapon.
define	_weapon_accuracy__02F,skill4; // The accuracy of the weapon (lower is better).
define	_weapon_vertex__02F,skill5;  	// The vertex on the turret model used as the firing point

define	_health__02F,skill6;				// damage it can take

// MOVE SKILLS
define	_move_pan__02F,skill7;			// Max angle the turret can pan in (if 360 or greater, no limit).
define	_move_tilt__02F,skill8;			// Max angle the turret can tilt in (if 360 or greater, no limit).
define	_move_speed__02F,skill9;		// Speed at which the turret can rotate.
// SCAN SKILLS
define	_scan_range__02F,skill10;		// Max distance that the turret can scan for the player.
define	_scan_pan__02F,skill11;			// Max pan angle the turret scans for the player in (if 360 or greater, no limit).
define	_scan_tilt__02F,skill12;		// Max tilt angle the turret scans for the player in (if 360 or greater, no limit).
define	_reaction__02F,skill13;			// "delay" between scanning the player and reacting to them.


// INTERNAL
define	_qAlert__02F,skill30;	// > 0 if knows about player
define	_weapon_cycle__02F,skill31;	// time (ticks) left to reload weapon
define	_target_pan__02F,skill35;	// angle to pan to
define	_target_tilt__02F,skill36;	// angle to tilt to
define	_target_roll__02F,skill37;	// angle to roll to (always zero!)


var	aifixed01_locked;	// used to count the number of angles "locked in" by a turret
								// (2 means that both pan and tilt are facing player)

////////////////////////////////////////////////////////////////////////
// Desc: Fire the turret weapon
//
function	AIFixed01_Turret_Fire(weaponOffset)
{
	// fire gun
	if(my._weapon_vertex__02F <= 0)
	{
		// use model center
		vec_set(weapons00_from_vec,my.x);
	}
	else
	{
		// use weapon vertex
		vec_for_vertex(weapons00_from_vec, my, my._weapon_vertex__02F);
	}

	// set to angle
	vec_set(weapons00_to_vec,my.pan);

	// fx comes from the same area shoot comes from
	vec_set(weapons00_fx_vec,weapons00_from_vec);


	// set reload cycle
	my._weapon_cycle__02F = my._weapon_reload__02F * 16;	// convert to ticks

	YOU = NULL;	// otherwise the player may be set as YOU

	if(weaponOffset > 0)
	{
		// fire weapon with param: accuracy,range,damage,gun_fx
		Weapons00_Gun_Fire_AngleA( my._weapon_accuracy__02F,
							  	  			weapons00_template[weaponOffset + weapons00_weap_range],
							  	  			weapons00_template[weaponOffset + weapons00_weap_damage],
							  	  			weapons00_template[weaponOffset + weapons00_weap_fx] );
	}
	else
	{
		// launch shell
		weapLaunch00_shell_data[0] = weapLaunch00_template[weaponOffset + weapLaunch00_weap_speed]; // launch speed
		weapLaunch00_shell_data[1] = weapLaunch00_template[weaponOffset + weapLaunch00_weap_grav];  // gravity effect
		weapLaunch00_shell_data[2] = weapLaunch00_template[weaponOffset + weapLaunch00_weap_damage];// base damage
		weapLaunch00_shell_data[3] = weapLaunch00_template[weaponOffset + weapLaunch00_weap_radius];// blast radius

		WeapLaunch00_Fire_AngleA(weapLaunch00_template[weaponOffset + weapLaunch00_weap_shell]);


		// play weapon launch noise
		WeaponFX00_Play_Shot(plBiped01_entity,weapLaunch00_template[weaponOffset + weapLaunch00_weap_snd]);

		// play launching fx
	//++	Weapons00_Gun_FX(weap_fx);
	}


}


// Desc: given a target angle, calc movement for this turn to reach that goal
function	AIFixed01_Turret_Towards(tar_ang,cur_ang)
{
	tar_ang = ang(tar_ang-cur_ang);

	// turn to face player
	if(tar_ang > my._move_speed__02F*TIME)
	{
		// turn anti-clockwise
		return(my._move_speed__02F*TIME);
	}
	else
	{
		if(tar_ang < -my._move_speed__02F*TIME)
		{
			// turn anti-clockwise
 			return(-my._move_speed__02F*TIME);
		}
		else
		{
			// turn "exactly" to
			aifixed01_locked += 1;	// used to tell the turret the player is locked in this angle
			return(tar_ang);
		}


	}
	return(0);	// should never get here!
}



// Desc: Event handler for the turret
function	AIFixed01_Turret_Event()
{
	if((EVENT_TYPE == event_shoot) && (gid01_event_type == gid01_event_projectile_damage))
	{
		my._health__02F -= gid01_event_value;
	}
}

// Desc: Update loop for the turret
//
// skill40: used to store range to player
// skill41: used to store weapon array offset
function	AIFixed01_Turret_Update()
{
	my._weapon_cycle__02F = 0;	// start loaded

	while(ME != NULL)
	{
		while( plSelect_curr_ent == NULL) { wait(1); }	// don't do anything until we have a player_ent in level

		///////////////////////////////////////////
		// handle changes that happen each frame //
		my._weapon_cycle__02F -= time;



		// If knows about the player
		if(my._qAlert__02F > 0)
		{
			//////////////////////////////
			// Always turn to face player
			// get the target angle to face player
			vec_set(temp,plSelect_curr_ent.x);
			vec_sub(temp,my.x);
			vec_to_angle(my._target_pan__02F,temp); // _target_pan/tilt/roll is angle to player

			aifixed01_locked = 0;	// reset locked count (modified in AIFixed01_Turret_Towards())
			// turn to face player
   		temp.pan = AIFixed01_Turret_Towards(my._target_pan__02F,my.pan);
	  		temp.tilt = AIFixed01_Turret_Towards(my._target_tilt__02F,my.tilt);
			temp.roll = 0;
			move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box;
			c_rotate(my,temp.pan,move_mode);

			// Still in sight?
			// check ray between player and myself (ignore LOS, since we have
			//at least some idea where they are at).
			trace_mode = IGNORE_ME + IGNORE_PASSABLE + IGNORE_PASSENTS;
 			my.skill40 = c_trace(my.x,plSelect_curr_ent.x,trace_mode);
	  		if((you == plSelect_curr_ent))// && (my.skill40 <= my._scan_range__02F))
			{
				my._qAlert__02F = 6*16;	// interest in player at maximum    +++

				if(my._weapon_cycle__02F <= 0)  // weapon ready?
				{
					// if player is in range & angle of weapon
					if( aifixed01_locked > 1 )	// at least two angles locked
					{
						// get offset
						if(my._weapon_index__02F < 100)
						{
							my.skill41 = (my._weapon_index__02F*weapons00_weap_field_count);
							if(my.skill40 <= weapons00_template[my.skill41 + weapons00_weap_range])
							{
								// shoot at player
								AIFixed01_Turret_Fire(my.skill41);
							}
						}
						else	// use weapLaunch00 template
						{
							my.skill41 = -((my._weapon_index__02F-100)*weapLaunch00_weap_field_count);
							// shoot at player
 							AIFixed01_Turret_Fire(my.skill41);
 						}
					}
				}


			}
			else	// can't see them anymore!
			{
				// slowly lose interest in player
				my._qAlert__02F -= time;
			}
		}
		else // does not know about player
		{
			// scan for player

			// if player found
			if(1)
			{
				// set alert value
				my._qAlert__02F = 64;      //+++
			}

		}



		wait(1);
	}
}

// action: AIFixed01_Turret
// title: Gun Turret
// Desc: Fixed weapon turret. Scans for player and fires on them.
//
// uses:	_weapon_index__02F,_weapon_ammo__02F,_weapon_reload__02F,_weapon_accuracy__02F,_weapon_vertex__02F, _health__02F
//	uses:	_move_pan__02F,_move_tilt__02F,_move_speed__02F,_scan_range__02F,_scan_pan__02F,_scan_tilt__02F,_reaction__02F
//
//------------------
// section: Weapon
// help: Select the type of weapon this turret uses.
// cntl: spin 1 x 1
// skill1: WeaponID 1
//	help: Number of shots (-1 for unlimited)
// cntl: spin -1 x 1
//	skill2: Ammo -1
//	help: Number of seconds to reload the weapon.
// skill3: Reload 2
// help: The accuracy of the weapon (lower is better).
//	skill4: Accuracy 5
// help: The vertex on the turret model used as the firing point
//	help: (0: use model's centerpoint)
//	skill5: WeapVertex 0
//
// help: How much damage this turret can take (-1, unlimited)
// skill6: Health 50
//------------------
// section: Movement
// help: Max angle the turret can pan in (if 360 or greater, no limit).
// skill7: Pan 360
// help: Max angle the turret can tilt in (if 360 or greater, no limit).
// skill8: Tilt 60
// help: Speed at which the turret can rotate (0, does not rotate).
// skill9: Speed 1
//------------------
// section: Scan
// help: Max distance that the turret can scan for the player.
// skill10: ScanRange 2000
// help: Max pan angle the turret scans for the player in (if 360 or greater, no limit).
// skill11: ScanPan 360
// help: Max tilt angle the turret scans for the player in (if 360 or greater, no limit).
// skill12: ScanTilt 60
// help: "delay" between scanning the player and reacting to them.
// skill13: Reaction 0.25
action	AIFixed01_Turret
{
ifdef	GID01_OLD_COLLIDE;	// use the old collision code
	my.fat = OFF; my.narrow = ON;
ifelse;	// use A6 collison
	my.fat = ON; my.narrow = ON;
	c_setminmax(my);
endif;


	my._gid01_id = gid01_ai_const;	// set ID to generic AI type

	/////////////////////////////////////////////////////
	// Setup event handler
	my.event = AIFixed01_Turret_Event;	// handle events
	my.enable_shoot = on;  	// gun shoot
	my.enable_scan = on;		// explosions, alerts

	/////////////////////////////////////////////////////
	// INIT WEAPON
	my._weapon_index__02F = int(my._weapon_index__02F - 1);  // my weaponid offset for index
	if(my._weapon_index__02F < 0) { my._weapon_index__02F = 0; } // id of weapon used
	else
	{
		if(my._weapon_index__02F >= weapons00_weap_count) { my._weapon_index__02F = weapons00_weap_count-1; }
	}
	// weapon vertex
	if(my._weapon_vertex__02F <= 0) { my._weapon_vertex__02F = -1; }  // use model origin
	// weapon accuracy
	if(my._weapon_accuracy__02F <= 0) { my._weapon_accuracy__02F = 0; }

	////////////////////////////////////////////////////////
	// Health
	if(my._health__02F == -1)
	{
		my.event = null; // take no damage
	}

	////////////////////////////////////////////////////////
	// MOVEMENT
	if(my._move_pan__02F > 360) { my._move_pan__02F = 360; }
	if(my._move_pan__02F < 0) 	 { my._move_pan__02F = 0; }
	if(my._move_tilt__02F > 360) { my._move_tilt__02F = 360; }
	if(my._move_tilt__02F < 0)   { my._move_tilt__02F = 0; }
	if(my._move_speed__02F < 0) { my._move_speed__02F = 0; }

	////////////////////////////////////////////////////////
	// SENSES
	if(my._scan_range__02F < 0) { my._scan_range__02F = 0; }
	if(my._scan_pan__02F > 360) { my._scan_pan__02F = 360; }
	if(my._scan_pan__02F < 0) 	 { my._scan_pan__02F = 0; }
	if(my._scan_tilt__02F > 360) { my._scan_tilt__02F = 360; }
	if(my._scan_tilt__02F < 0)   { my._scan_tilt__02F = 0; }

	if(my._reaction__02F <= 0) { my._reaction__02F = 0; }	// reaction time


	// Call the update loop
	AIFixed01_Turret_Update();
}



endif;	// aiFixed_wdl