ifndef cameraFX_wdl;
define cameraFX_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
//
// title: Camera FX (00)
// class: CAMERA
// type: USER
// image: cameraFX00.pcx
// help: Allows you to adjust camera view parameters (arc, aspect, fog, etc.) in game by using triggers.
//
// note: Make sure to create another trigger to reset the value as needed.
//
// needs: gid01.wdl, display00.wdl
//
// external:
//
// prefix: cameraFX00_
// idcode: 024
//
// ENDHEADER
// ---------------------------------------------------------------------


// "target" skills
define	_ambient__024,SKILL1;   // -1 to turn off, 1-100 set level
define	_arc__024,SKILL2;
define	_aspect__024,SKILL3;
define	_clip_far__024,SKILL4;
define	_clip_near__024,SKILL5;
define	_clip_factor__024,SKILL6;
define	_clip_particles__024,SKILL7;
define	_clip_size__024,SKILL8;
define	_fog_color__024,SKILL9;	// -1 to turn off fog, 1-5 to select fog
define	_fog_start__024,SKILL10;
define	_fog_end__024,SKILL11;

define	_reset_time__024,SKILL12; 	// Time between activation events (seconds).
define	_fade_time__024,SKILL13;  	// Time to fade from current camera settings to this new setting (seconds).


define	_curr_time__024,SKILL21;	// Current time elapsed. 0 is not active

define	_remove__024,FLAG1;	// Remove after triggered?


// Desc: used to fade the ambient value
function	CameraFX00_Fade_Ambient(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.ambient)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.ambient += delta*TIME;
	}

	// snap to end.
	camera.ambient = target;
}


// Desc: used to fade the camera arc
function	CameraFX00_Fade_Arc(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.arc)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.arc += delta*TIME;
	}

	// snap to end.
	camera.arc = target;
}

// Desc: used to fade the camera aspect ratio
function	CameraFX00_Fade_Aspect(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.aspect)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.aspect += delta*TIME;
	}

	// snap to end.
	camera.aspect = target;
}

// Desc: used to fade the camera clip near range
function	CameraFX00_Fade_ClipNear(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.clip_near)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.clip_near += delta*TIME;
	}

	// snap to end.
	camera.clip_near = target;
}
// Desc: used to fade the camera clip far range
function	CameraFX00_Fade_ClipFar(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.clip_far)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.clip_far += delta*TIME;
	}

	// snap to end.
	camera.clip_far = target;
}
// Desc: used to fade the clip_factor
function	CameraFX00_Fade_ClipFactor(target)
{
	var	delta;	// diff per tick
	delta = (target - clip_factor)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		clip_factor += delta*TIME;
	}

	// snap to end.
	clip_factor = target;
}
// Desc: used to fade the clip_particles
function	CameraFX00_Fade_ClipParticles(target)
{
	var	delta;	// diff per tick
	delta = (target - clip_particles)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		clip_particles += delta*TIME;
	}

	// snap to end.
	clip_particles = target;
}
// Desc: used to fade the clip_size value
function	CameraFX00_Fade_ClipSize(target)
{
	var	delta;	// diff per tick
	delta = (target - clip_size)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		clip_size += delta*TIME;
	}

	// snap to end.
	clip_size = target;
}


// Desc: used to fade the camera fog staring value
function	CameraFX00_Fade_FogStart(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.fog_start)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.fog_start += delta*TIME;
		// make sure the fog "starts" before it "ends"
		if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
	}

	// snap to end.
	camera.fog_start = target;
	// make sure the fog "starts" before it "ends"
	if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
}
// Desc: used to fade the camera fog ending value
function	CameraFX00_Fade_FogEnd(target)
{
	var	delta;	// diff per tick
	delta = (target - camera.fog_end)/my._fade_time__024;

 	// NOTE: my._curr_time is update in calling function (CameraFX00_Fade_Event())
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		camera.fog_end += delta*TIME;
		// make sure the fog "starts" before it "ends"
		if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
	}

	// snap to end.
	camera.fog_end = target;
	// make sure the fog "starts" before it "ends"
	if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
}








// Desc: fade between the current camera fx and the target.
function CameraFX00_Fade_Event()
{
	// make sure this is the player
	if(YOU != plSelect_curr_ent) { return; }
	// don't activate if already running
	if(my._curr_time__024 != 0) { return; }

	// Ambient
	if(my._ambient__024 != 0)
	{
		if(my._ambient__024 < 0) { CameraFX00_Fade_Ambient(0); }
		else { CameraFX00_Fade_Ambient(my._ambient__024); }
	}

	// Camera Arc
	if((my._arc__024 > 0) && (my._arc__024 <= 180)) { CameraFX00_Fade_Arc(my._arc__024); }

	// Aspect Ratio
	if( (my._aspect__024 != 0)
	 && (my._aspect__024 <= 2)
	 && (my._aspect__024 >= -2) ) { CameraFX00_Fade_Aspect(my._aspect__024); }

	// clip
	if(my._clip_near__024 >= 1) { CameraFX00_Fade_ClipNear(my._clip_near__024); }
	if(my._clip_far__024 > camera.clip_near) { CameraFX00_Fade_ClipFar(my._clip_far__024); }
	if(my._clip_factor__024 > 0) { CameraFX00_Fade_ClipFactor(my._clip_factor__024); }
	if((my._clip_particles__024 > 0) && (my._clip_particles__024 < 1)) { CameraFX00_Fade_ClipParticles(my._clip_particles__024); }
	if((my._clip_size__024 > 0) && (my._clip_size__024 < 1)) { CameraFX00_Fade_ClipSize(my._clip_size__024); }

	// fog
	if(my._fog_color__024 != 0)
	{
		// note! can't fade here!
		if(my._fog_color__024 < 0) { my._fog_color__024 = 0; }
		fog_color = my._fog_color__024;
	}

	if(my._fog_start__024 > 0)
	{
		CameraFX00_Fade_FogStart(my._fog_start__024);
	}
	if(my._fog_end__024 > 0)
	{
		CameraFX00_Fade_FogEnd(my._fog_end__024);
	}

	// decrease the counter...
	while(my._curr_time__024 < my._fade_time__024)
	{
		wait(1);	// once a frame
 		my._curr_time__024 += TIME;
	}

	// remove on finish flag set?
	if(my._remove__024 == ON) { wait(1); ent_remove(me); return; }


	// wait for the reset time before allowing the event to run again..
	if(my._reset_time__024 > 0)
	{
		sleep(my._reset_time__024);
	}
	my._curr_time__024 = 0; // flag this as no longer running
}

// Desc: trigger a camera fx instantly when touched.
function CameraFX00_Snap_Event()
{
	// make sure this is the player
	if(YOU != plSelect_curr_ent) { return; }
	// don't activate if already running
	if(my._curr_time__024 != 0) { return; }

	my._curr_time__024 = 1; // flag this as running

	// Ambient
	if(my._ambient__024 != 0)
	{
		if(my._ambient__024 < 0) { my._ambient__024 = 0; }
		camera.ambient = my._ambient__024;
	}

	// Arc
	if((my._arc__024 > 0) && (my._arc__024 <= 180)) { camera.arc = my._arc__024; }

	// Aspect
	if( (my._aspect__024 != 0)
	 && (my._aspect__024 <= 2)
	 && (my._aspect__024 >= -2) ) { camera.aspect = my._aspect__024; }

	// clipping
	if(my._clip_near__024 >= 1) { camera.clip_near = my._clip_near__024; }
	if(my._clip_far__024 > camera.clip_near) { camera.clip_far = my._clip_far__024; }
	if(my._clip_factor__024 > 0) { clip_factor = my._clip_factor__024; }
	if((my._clip_particles__024 > 0) && (my._clip_particles__024 < 1)) { clip_particles = my._clip_particles__024; }
	if((my._clip_size__024 > 0) && (my._clip_size__024 < 1)) { clip_size = my._clip_size__024; }

	// fog
	if(my._fog_color__024 != 0)
	{
		if(my._fog_color__024 < 0) { my._fog_color__024 = 0; }
		fog_color = my._fog_color__024;
	}

	if(my._fog_start__024 > 0)
	{
		camera.fog_start = my._fog_start__024;
		if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
	}
	if(my._fog_end__024 > 0)
	{
		camera.fog_end = my._fog_end__024;
		if(camera.fog_end < camera.fog_start) { camera.fog_end = camera.fog_start; }
	}



	// remove on finish?
	if(my._remove__024 == ON) { wait(1); ent_remove(me); return; }

	if(my._reset_time__024 > 0)
	{
		sleep(my._reset_time__024);
	}
	my._curr_time__024 = 0; // flag this as no longer running

}


// Desc: Activates a camera Fx when touched.
//
// uses: _ambient__024, _arc__024, _aspect__024,
// uses: _clip_far__024, _clip_near__024
// uses: _clip_factor__024, _clip_particles__024, _clip_size__024
// uses: _fog_color__024, _fog_start__024, _fog_end__024
// uses: _reset_time__024, _fade_time__024
//
// uses: _remove__024
action CameraFX00_Trigger
{
//!!! note: placeholder until enable_enter is added.
	my.passable = on; my.invisible = off;
	my.transparent = on; my.alpha = 50;     //!!!!!!!!!

 	my.trigger_range = 50;   //!!!!!!!
 	my.enable_trigger = on;  //!!!!!!!

	// Set up the reset timer
	if(my._reset_time__024 == 0) { my._reset_time__024 = 5; }   // 5 second default
	else { if(my._reset_time__024 < 0) { my._reset_time__024 = 0; } } // allow the user to have no reset delay


	if(my._fade_time__024 <= 0)
	{
		my.event = CameraFX00_Snap_Event; // do the fx without delay
	}
	else
	{
		my._fade_time__024 *= 16;	// convert to "ticks"
		my.event = CameraFX00_Fade_Event; // fade from current to target
	}
//--	my.enable_impact = on;
}

endif; // cameraFX00_wdl