ifndef cameraOrbit_wdl;
define cameraOrbit_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Orbit Camera
// class: CAMERA
// type: USER
// image: cameraOrbit01.pcx
// help: The Orbit Camera is an external camera that follows the current "camera target".
// help: The camera's inital pan and height values can be set. These values can be
// help:changed by the player during the game.
//
// uses: display_active_camera_vec
// needs: display00.wdl, miscInput01.wdl
//
// needs: cameraSelect.wdl, cameraTarget.wdl
//
// prefix: cameraOrbit01_
// idcode: 007
//
// ENDHEADER
// ---------------------------------------------------------------------



// General Camera Entries



// entry: ID
// help: Identification number for this camera (used in cameraSelect)
// help: NOTE: Action not defined if two camera share a single value
// id: 2
var	cameraOrbit01_id = 3;

// enable: Hide Focus when Inside?
// id: 1
define	cameraOrbit01_hide_focus_b;


// enable: Make orbit camera selectable by user?
// id: 3
define	cameraOrbit01_usr_selectable_b;


// enable: Preform collision test?
// id: 4
define	cameraOrbit01_collide_b;

// enable: Is this an "avatar" camera?
// help: Avatar cameras are cameras who's source/pan are those of the player's eyes
// id: 20
//define	cameraOrbit01_avatar_q;


// --------------------------------------------------------------
// section: Follow Distance

// entry: Starting distance
// id: 5
var cameraOrbit01_dist = 150;
// entry: Maximum Distance
// id: 6
var cameraOrbit01_dist_max = 500;
// entry: Minimum Distance
// id: 7
var cameraOrbit01_dist_min = 25;

// entry: Starting height
// id: 8
var cameraOrbit01_height = 100;
// entry: Maximum height
// id: 9
var cameraOrbit01_height_max = 500;
// entry: Minimum height
// id: 10
var cameraOrbit01_height_min = -50;

// entry: Starting orbit rotation
// id: 11
var	cameraOrbit01_rotation = 0;

// -------------------------------------------------------------
// section: Input keys

// entry: Orbit camera rotate clockwise key
// id: 12
string cameraOrbit01_cw_key_c = "J";
var	cameraOrbit01_cw_scancode = 0;
// entry: Orbit camera rotate counter-clockwise key
// id: 13
string cameraOrbit01_ccw_key_c = "L";
var	cameraOrbit01_ccw_scancode = 0;
// entry: Orbit camera zoom in key
// id: 14
string cameraOrbit01_zoom_in_key_c = "I";
var	cameraOrbit01_zoom_in_scancode = 0;
// entry: Orbit camera zoom out key
// id: 15
string cameraOrbit01_zoom_out_key_c = "K";
var	cameraOrbit01_zoom_out_scancode = 0;
// entry: Orbit camera increase height key
// id: 16
string cameraOrbit01_inc_height_key_c = "U";
var	cameraOrbit01_inc_height_scancode = 0;
// entry: Orbit camera decrease height key
// id: 17
string cameraOrbit01_dec_height_key_c = "O";
var	cameraOrbit01_dec_height_scancode = 0;


// Local variables
var	cameraOrbit01_pos_vec[3];	// x,y,z postion of orbit camera
var	cameraOrbit01_ang_vec[3];	// pan,tilt,roll angles of the orbit camera
var	cameraOrbit01_bit_check; 	// used in bit-compairs to check is this camera is active


/////////////////////////////////////////////////////////////////////////
// desc: Used to attach an orbiting camera to an entity
//
// help: To disable camera features (such as increasing/decreasing the camera
// help:height) simply assign 'nullvalues' to the key comands you don't want
// help:your players to have access to.
starter CameraOrbit01_Init
{
	var	updated_b;	// has the camera been update yet this frame?
	var	view_number;	// the current view being concidered

	diag("\nWDL-Loaded:cameraOrbit01.wdl");

	// init bit-field value used to check to see if this camera is active
	cameraOrbit01_bit_check = 1<<cameraOrbit01_id;

	ifdef cameraOrbit01_usr_selectable_b;
	//if(1 == cameraOrbit01_usr_selectable_b)
	//{
		// increase the camera select "max_camera" value if this value fall outside the current range
		if(cameraOrbit01_bit_check > cameraSelect_max_camera)
		{ cameraSelect_max_camera = cameraOrbit01_bit_check; }

		// add this camera type to the collection of valid cameras in camera select
		cameraSelect_valid_cameras |= cameraOrbit01_bit_check;
	//}
	endif;

	/////////////////////////////
	// Set up keyboard scancodes
	/////////////////////////////
	// use MiscInput01_Key_For_Str() to convert strings into scancodes
	// Rotate CW key
	str_cpy(miscInput01_temp_str,cameraOrbit01_cw_key_c);
	cameraOrbit01_cw_scancode = MiscInput01_Key_For_Str();
	// Rotate CCW key
	str_cpy(miscInput01_temp_str,cameraOrbit01_ccw_key_c);
	cameraOrbit01_ccw_scancode = MiscInput01_Key_For_Str();
	// Zoom in key
	str_cpy(miscInput01_temp_str,cameraOrbit01_zoom_in_key_c);
	cameraOrbit01_zoom_in_scancode = MiscInput01_Key_For_Str();
	// Zoom out key
	str_cpy(miscInput01_temp_str,cameraOrbit01_zoom_out_key_c);
	cameraOrbit01_zoom_out_scancode = MiscInput01_Key_For_Str();
	// Inc height key
	str_cpy(miscInput01_temp_str,cameraOrbit01_inc_height_key_c);
	cameraOrbit01_inc_height_scancode = MiscInput01_Key_For_Str();
	// Dec height key
	str_cpy(miscInput01_temp_str,cameraOrbit01_dec_height_key_c);
	cameraOrbit01_dec_height_scancode = MiscInput01_Key_For_Str();



	while(1)
	{
		updated_b = 0;	// camera hasn't been updated yet this frame
		view_number = 0;	// start with main view

		// for all active views
		while(view_number < display_active_views)
		{
			// check to see if this camera updates the current view
			if(display_active_camera_vec[view_number] & cameraOrbit01_bit_check)
			{
				// if the camera hasn't updated itself this frame...
				if(0 == updated_b)
				{
					// update camera position
					CameraOrbit01_Update();

					// camera has been updated this frame
					updated_b = 1;
				}

				// update current view
				CameraOrbit01_Update_View(view_number);

			}

			view_number +=1;
		}

		wait(1);	// every frame
	}

}


// desc: update the orbit camera
function CameraOrbit01_Update()
{
	// check and handle input
	if(Key_Pressed(cameraOrbit01_cw_scancode))
	{
  		cameraOrbit01_rotation += 5*TIME;
		cameraOrbit01_rotation = ang(cameraOrbit01_rotation); // keep in range -180 to 180
	}
	if(Key_Pressed(cameraOrbit01_ccw_scancode))
	{
  		cameraOrbit01_rotation -= 5*TIME;
		cameraOrbit01_rotation = ang(cameraOrbit01_rotation); // keep in range -180 to 180
	}
	if(Key_Pressed(cameraOrbit01_zoom_in_scancode))
	{
  		cameraOrbit01_dist -= 5*TIME;
		if(cameraOrbit01_dist < cameraOrbit01_dist_min) { cameraOrbit01_dist = cameraOrbit01_dist_min; }
	}
	if(Key_Pressed(cameraOrbit01_zoom_out_scancode))
	{
  		cameraOrbit01_dist += 5*TIME;
		if(cameraOrbit01_dist > cameraOrbit01_dist_max) { cameraOrbit01_dist = cameraOrbit01_dist_max; }
	}
	if(Key_Pressed(cameraOrbit01_inc_height_scancode))
	{
  		cameraOrbit01_height += 5*TIME;
		if(cameraOrbit01_height > cameraOrbit01_height_max) { cameraOrbit01_height = cameraOrbit01_height_max; }
	}
	if(Key_Pressed(cameraOrbit01_dec_height_scancode))
	{
  		cameraOrbit01_height -= 5*TIME;
		if(cameraOrbit01_height < cameraOrbit01_height_min) { cameraOrbit01_height = cameraOrbit01_height_min; }
	}


	// orbit the target
 	cameraOrbit01_pos_vec.x = cameraTarget_pos_vec.x + (cameraOrbit01_dist * SIN(cameraOrbit01_rotation));
 	cameraOrbit01_pos_vec.y = cameraTarget_pos_vec.y + (cameraOrbit01_dist * COS(cameraOrbit01_rotation));
 	cameraOrbit01_pos_vec.z = cameraTarget_pos_vec.z + cameraOrbit01_height;


	// handle collision
	ifdef cameraOrbit01_collide_b;
	//if(1 == cameraOrbit01_collide_b)
	//{
		// collide with any object in the way
		c_trace(cameraTarget_pos_vec.x,cameraOrbit01_pos_vec.x,(IGNORE_PASSABLE+IGNORE_PASSENTS+IGNORE_MODELS));
		if(trace_hit == 1)
		{
 			// offset from that target along the vector
			vec_set(cameraOrbit01_pos_vec.x,TARGET.X);
		}

	//}
	endif;

 	// face the target
 	temp.x = cameraTarget_pos_vec.x - cameraOrbit01_pos_vec.x;
   temp.y = cameraTarget_pos_vec.y - cameraOrbit01_pos_vec.y;
   temp.z = cameraTarget_pos_vec.z - cameraOrbit01_pos_vec.z;
 	vec_to_angle(temp,temp);
 	cameraOrbit01_ang_vec.PAN = temp.PAN;
 	cameraOrbit01_ang_vec.TILT = temp.TILT;

}


// desc: update the view whos ID is passed in to the position/orientation
//      of the Orbit Camera
function	CameraOrbit01_Update_View(view_number)
{
	if(view_number == 0)
	{
		// update main view (camera)
		vec_set(camera.x,cameraOrbit01_pos_vec.x);
		vec_set(camera.pan,cameraOrbit01_ang_vec.pan);

		// is this an avatar view?
		//cameraTarget_avatar_view_q = cameraOrbit01_avatar_q;
		ifdef cameraOrbit01_avatar_q;
			cameraTarget_avatar_view_q = 1;
		ifelse;
			cameraTarget_avatar_view_q = 0;
		endif;
	}

	if(cameraTarget_ent != NULL) { cameraTarget_ent.invisible = off; }


	ifdef cameraOrbit01_hide_focus_b;
	//if(1==cameraOrbit01_hide_focus_b)
	//{	// hide the target entity from view if inside
		camera.genius = cameraTarget_ent;
	//}
	ifelse;
	//else
	//{	// show the target entity no matter what
		camera.genius = 0;
	//}
	endif;

	// +++ other views


}

endif;