ifndef door_wdl;
define door_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Doors and Lids (01)
// class: OBJECT
// type: USER
// image: door01.pcx
// help: Make map entities into doors that react to activate scans.
//
// needs: gid01.wdl, plSelect.wdl, trigger00.wdl
//
// external:
//
// prefix: door01_
// idcode: 00B
//
// ENDHEADER
// ---------------------------------------------------------------------


// -----------------------------------------------------------------
// section: Sounds
// entry: Locked Door Sound
// entry_help: Sound played when a locked door is activated without a key.
// id: 1
sound door01_locked_snd = <locked.wav>;

// entry: Door Locked Sound Volume
// id: 2
define door01_locked_snd_vol = 50;

// entry: Door Opening Sound
// entry_help: Sound played when a door is activated.
// id: 3
sound door01_open_snd = <door_open.wav>;

// entry: Door Opening Sound Volume
// id: 4
define door01_open_snd_vol = 50;


// lift01 skills
define	_rot_pan__00B,  skill1;   	// ending rotation
define	_rot_tilt__00B, skill2;
define	_rot_roll__00B, skill3;
define	_speed__00B,    skill4;     // speed in deg/tick
define	_rest_time__00B,skill5;		// time spent "resting" at each point (ticks)

define	_push__00B, skill6; // "PUSH" value (note! overwriten by _startrot_pan__00B after use!)

define	_startrot_pan__00B, skill6;	// starting rotation
define	_startrot_tilt__00B,skill7;
define	_startrot_roll__00B,skill8;

define	_dist_pan__00B,skill9;			// distance between rotations
define	_dist_tilt__00B,skill10;
define	_dist_roll__00B,skill11;

define	_use_trigger__00B,skill12;		// id number of the trigger used to open/close door
define	_use_key__00B,skill13;			// requires a key to be picked up before opening


define	_time_to_target__00B,skill15;	// time remaining before hitting target rotation

define	_state__00B, skill20;			// current state:
define 	door01_state_at_start_const,1;    	// resting at start point
define 	door01_state_at_end_const,2;    		// resting at end point
define 	door01_state_towards_end_const,3;  	// moving towards the end point
define 	door01_state_towards_start_const,4;	// moving towards the start point
define 	door01_state_waiting_const,5;			// temporarily inactive


define	_use_absolute__00B,flag1;	// use absolute rotations
define	_oscillate__00B,   flag2;  // oscillate between the two points
define	_ignore_scan__00B, flag3;  // ignore scans
define	_use_sonar__00B,	flag4;   // activate on sonar
define	_ignore_click__00B,flag5;  // ignore left mouse click
define 	_no_sound__00B,flag8;		// don't make a sound


var  door01_key_bitlist;	// "bitlist" for door keys

////////////////////////////////////////////////////////////////////////
// Functions




function Door01_Activate()
{
	// calculate the directional speed..
	if(door01_state_at_start_const == my._state__00B)
	{
		// move towards end

		// calculate delta per tick
		vec_set(my._gid01_ang_vel_pan,my._dist_pan__00B);
		if(my._speed__00B <= 0) { my._speed__00B = 1; }
		vec_normalize(my._gid01_ang_vel_pan,my._speed__00B);	// delta per tick

		my._state__00B = door01_state_towards_end_const;
	}
	else
	{
		if(door01_state_at_end_const == my._state__00B)
		{
			// move towards start

			// calculate delta per tick
			vec_set(my._gid01_ang_vel_pan,my._dist_pan__00B);
			if(my._speed__00B <= 0) { my._speed__00B = 1; }
			vec_normalize(my._gid01_ang_vel_pan,my._speed__00B);	// delta per tick
			vec_scale(my._gid01_ang_vel_pan,-1);	// move the "other way"

			my._state__00B = door01_state_towards_start_const;
		}
		else	// not at the start OR end postion (either moving or resting)
		{
			return;
		}
	}

	if(my._no_sound__00B == OFF)
	{
		// play door active sound
		ent_playsound(my,door01_open_snd,door01_open_snd_vol);
	}

	// calculate "tick to target"
	my._time_to_target__00B = vec_length(my._dist_pan__00B)/my._speed__00B;


	wait(1);	// wait a frame before movement

	while(my._time_to_target__00B > time)
	{
		my._time_to_target__00B -= time;
		// temp is this frame's movement vector
		vec_set(temp.pan,my._gid01_ang_vel_pan);
		vec_scale(temp.pan,time);

		vec_add(my.pan,temp.pan);	// rotate
		wait(1);	// move every frame
	}

	// snap to end point
	if(door01_state_towards_end_const == my._state__00B)
	{
		// end rotation
		vec_set(my.pan,my._rot_pan__00B);

		// wait for pre-set amount of time
		my._state__00B = door01_state_waiting_const;
		sleep(my._rest_time__00B);

		my._state__00B = door01_state_at_end_const;
	}
	else
	{
		// snap to start
		vec_set(my.pan,my._startrot_pan__00B);

		// wait for pre-set amount of time
		my._state__00B = door01_state_waiting_const;
		sleep(my._rest_time__00B);

		my._state__00B = door01_state_at_start_const;
	}
}

function	Door01_Event()
{
	// only activate on "activate" scans
	if((event_scan == event_type)
		&& (gid01_event_scan_activate != gid01_event_type))
	{
		return;
	}

	if(my._use_key__00B > 0)
	{
		// check for key
		if(Trigger00_Check(my._use_key__00B,door01_key_bitlist) != 1)
		{
			if(my._no_sound__00B == OFF)
			{
				// play door locked sound
				ent_playsound(my,door01_locked_snd,door01_locked_snd_vol);
			}

			return;
		}
	}

	Door01_Activate();
}

// Desc: check the triggers to see if we've been tapped
function Door01_Check_Trigger()
{
	while(1)
	{
		wait(1);
		if(Trigger00_Check(my._use_trigger__00B,trigger00_bitlist) == 1)
		{
			Door01_Activate(); // first activate
			while(Trigger00_Check(my._use_trigger__00B,trigger00_bitlist) == 1) { wait(1); }
			Door01_Activate(); // return..
		}
	}

}


////////////////////////////////////////////////////////////////////////
// Actions

// action: Door01
// title: Door
// Desc: A door entity will pivot on its origin point.
//
// Defaults: Door that pans 90o when clicked or "activate" scanned.
//
// uses _rot_pan__00B, _rot_tilt__00B, _rot_roll__00B,
// uses _use_absolute__00B, _speed__00B, _oscillate__00B,
// uses _push__00B
// uses _ignore_scan__00B, _ignore_click__00B, _use_sonar__00B
// uses _rest_time__00B, _use_trigger__00B, _use_key__00B
//
//------------------
// section: Target Pos/Angle
//
//	skill1: Pan 90
//	skill2: Tilt 0
//	skill3: Roll 0
// skill4: Speed 1
// skill5: RestTime 2
//
// help: Use absolute rotation?
// flag1: Absolute? 0
//
// flag2: Oscillate? 0
//
// -----------------
// section: Activate
// help: If 0, this door doesn't use a trigger
// skill12: Trigger 0
//
// help: If 0, this door doesn't require a key
// skill13: Key 0
//
// flag3: IgnoreScan 0
//
// flag4: UseSonar 0
//
// flag5: IgnoreClick 0
//
//
// ------------------
// section: Misc
// help: "PUSH" value (note! overwriten by _startrot_pan__00B after use!)
// skill6: Push 10
//
// flag8: NoSound 0
action	Door01
{
	// identify ourselves globally as a door
	my._gid01_id = gid01_door_const;

	// set up events
	my.event = Door01_Event;
	if( 0 == my._ignore_scan__00B) { my.enable_scan = on; }
	if( 1 == my._use_sonar__00B) { my.enable_sonar = on; }
	if( 0 == my._ignore_click__00B) { my.enable_click = on; }

	if(my._push__00B == 0)
	{
		my.push = 10;
	}
	else
	{
		my.push = my._push__00B;
	}

	// NOTE!  my._push__00B is no longer valid past this point!!!

	// set up door rotation (origin and target)
	if((my._rot_pan__00B == 0) && (my._rot_tilt__00B == 0) && (my._rot_roll__00B == 0))
	{
		my._rot_pan__00B = 90;	// default iff all values equal 0
	}
	if(my._speed__00B == 0)
	{
		my._speed__00B = 1;	// default speed of rotation
	}

	vec_set(my._startrot_pan__00B,my.pan);  // save start point
	if(0 == my._use_absolute__00B)
	{
		vec_set(my._dist_pan__00B,my._rot_pan__00B); // save pan distance
		vec_add(my._rot_pan__00B,my.pan);       // calculate end point
	}
	else
	{
		// use the rotation values given & calculate the distance between.
		vec_diff(my._dist_pan__00B,my._rot_pan__00B,my._startrot_pan__00B);
	}


	my._state__00B = door01_state_at_start_const;	// start at starting rotation

	// do we use a trigger?
	if(my._use_trigger__00B > 0)  { Door01_Check_Trigger(); }

}


//----------------------------------------------------------------------
// Keys

// Desc: event handler for key object
function	Door01_Key_Event()
{
	// only activate on "activate" scans
	if( (event_scan == event_type)
	  &&(gid01_event_scan_activate != gid01_event_type))
	{
		return;
	}

	// sets the key bitlist value
	Trigger00_Set(my._id__022,door01_key_bitlist);

	//++ play pickup noise


	// remove from level
	my.passable = on;
	my.event = NULL;
	my.enable_scan = off;
	my.enable_click = off;
	my.enable_sonar = off;
	my.enable_impact = off;
	wait(1);
	ent_remove(me);
}

// action: Door01_Key
// title: Door Key
// Desc: Door key object
// note: uses skill defines from trigger00.wdl
// uses: _id__022, _recycle_time__022
// uses: _ignore_scan__022, _ignore_click__022
action Door01_Key
{
	// cap value to min/max
	if(my._id__022 < 1)  { my._id__022 = 1; }
	if(my._id__022 > 20) { my._id__022 = 20; }

	// set up events
	my.event = Door01_Key_Event;
	if( 0 == my._ignore_scan__022) { my.enable_scan = on; }
	if( 0 == my._ignore_click__022) { my.enable_click = on; }
	my.enable_impact = on; // player hits
	my.enable_sonar = on; // player steps on
}


endif;