ifndef forcefield_wdl;
define forcefield_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Force Fields, Damage Fields, and Water (01)
// class: OBJECT
// type: USER
// image: forcefield01.pcx
// help: Make map entities into water, 'force fields', or 'damage fields' that effect the movement bipeds and vehicles.
//
// needs: gid01.wdl
//
// external:
//
// prefix: forcefield01_
// idcode: 00C
//
// ENDHEADER
// ---------------------------------------------------------------------


// forcefield01 skills
define	_alpha__00C,  skill1;   	// transparency
define	_vel_x__00C,skill2;   	// velocity in quants per tick
define	_vel_y__00C,skill3;   	//
define	_vel_z__00C,skill4;   	//

define	_damage__00C, skill2;	// damage per second



// Desc: Set up the common values of "Forcefield01" and "Forcefield01_Water"
function	Forcefield01_Setup
{
	my.passable = ON;
	my.transparent = ON;
	if(my._alpha__00C == 0)
	{
		my.alpha = 75;
	}
	else
	{
		my.alpha = my._alpha__00C;
	}

	vec_set(my._gid01_vel_x,my._vel_x__00C);     // set velocity
}


// action: Forcefield01
// title: Force Field
// Desc: Turns a map entity into a generic "Force Field"
//
// uses: _alpha__00C, _vel_x__00C, _vel_y__00C,
// uses: _vel_z__00C
//
// skill1: Alpha 75
// skill2: VelX 0
// skill3: VelY 0
// skill4: VelZ 0
//
action	Forcefield01
{
	my._gid01_id = gid01_forcefield_const;
	Forcefield01_Setup();
}

// action: Forcefield01_Water
// title: Water Field
// Desc: Turns a map entity into a body of water.
//
// uses: _alpha__00C, _vel_x__00C, _vel_y__00C,
// uses: _vel_z__00C
//
// skill1: Alpha 75
// skill2: VelX 0
// skill3: VelY 0
// skill4: VelZ 0
//
action	Forcefield01_Water
{
	my._gid01_id = gid01_water_const;
	Forcefield01_Setup();
}

// action: Forcefield01_Damage
// title: Damage Field
// Desc: Turns a map entity into a body of water.
//
// uses: _alpha__00C, _damage__00C
//
// skill1: Alpha 75
// skill2: Damage 0
action	Forcefield01_Damage
{
	my._gid01_id = gid01_damagefield_const;
	my._gid01_vel_x = my._damage__00C / 16; // convert to ticks and store in global vel_x
	Forcefield01_Setup();
}


endif;