ifndef fxa_wdl;
define fxa_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: fxa (01)
// class: ENVIR
// type: USER
// image: fxa01.pcx
// help: Effect library
//
// needs: gid01.wdl, trigger00.wdl, particle00.wdl
//
// external:
//
// prefix: fxa01_
// idcode: 02E
//
// Dev Note: some of these scripts are modified version of code writen by Dan Silverman
//
// ENDHEADER
// ---------------------------------------------------------------------



// ---------------------------------------------------------------------
// Lights

// action: FXA_LightBlink
// title: Blinking Light
// Desc: Cycle a dynamic light with 4 ranges and 4 delays.
// Desc: Can use trigger to only active light when trigger is set.
// Desc: (Negitive trigger, light is active when trigger is not set).
//
// help: If 0, no trigger is needed (always on)
// help: Otherwise, the light only blinks when the matching trigger is set.
// help: If the value is negitive, the light only blicks when the matching trigger is NOT set.
// skill12: Trigger 0
// section: Light Color
// skill1: Red 200
// skill2: Green 200
// skill3: Blue 200
// section: First Range
// help: Range in quants
// skill4: Range1 200
// help: Delay in seconds
// skill5: Delay1 3
// section: Second Range
// help: Range in quants
//	skill6: Range2 175
// help: Delay in seconds
// skill7: Delay2 1
// section: Third Range
// help: Range in quants
//	skill8: Range3 300
// help: Delay in seconds
// skill9: Delay3 1.25
// section: Fourth Range
// help: Range in quants
//	skill10: Range4 175
// help: Delay in seconds
// skill11: Delay4 1
// help: randomize time?
// flag1: Randomize? 1
action FXA_LightBlink
{
	my.red = my.skill1;
	my.green = my.skill2;
	my.blue = my.skill3;

	if(my.skill12 < 0)
	{
		// negitive trigger action
		my.skill12 = abs(my.skill12);
		my.skill13 = 0;	// on if trigger is not set!
	}
	else
	{
 		my.skill13 = 1;	// on if trigger is set!
	}
 	// cap trigger value to max
	if(my.skill12 > (trigger00_max_value_const*trigger00_bitlist_size_const))
	{
		my.skill12 = (trigger00_max_value_const*trigger00_bitlist_size_const);
	}

	while(1)
	{
		if((my.skill12 == 0)
			|| (Trigger00_Check(my.skill12,trigger00_bitlist) == my.skill13))   // trigger active/disactive?
		{
				// note: must finish cycle even if trigger is turned off!

				if(my.skill5 > 0)
				{
					my.lightrange = my.skill4; // lightrange 1
					if(my.flag1 == on) { sleep(random (my.skill5)-(my.skill5/2)); }
					else { sleep(my.skill5); }
				}
				if(my.skill7 > 0)
				{
					my.lightrange = my.skill6; // lightrange 2
					if(my.flag1 == on) { sleep(random (my.skill7)-(my.skill7/2)); }
					else { sleep(my.skill7); }
				}
				if(my.skill9 > 0)
				{
					my.lightrange = my.skill8; // lightrange 3
					if(my.flag1 == on) { sleep(random (my.skill9)-(my.skill9/2)); }
					else { sleep(my.skill9); }
				}
				if(my.skill11 > 0)
				{
					my.lightrange = my.skill10;// lightrange 4
					if(my.flag1 == on) { sleep(random (my.skill11)-(my.skill11/2)); }
					else { sleep(my.skill11); }
				}
			}
			else
			{
				// light off
				my.lightrange = 0;
				sleep(0.5);
			}
		}
	}
}

// ---------------------------------------------------------------------
// Misc


// action: FXA_Transparent
// title: Transparent Entity
// Desc: Make an entity transparent
//
//	help: The alpha value for this entity (0-100)
// cntl: spin 0 100 1
// skill1: Alpha 25
action FXA_Transparent
{
	MY.TRANSPARENT = ON;
	my.alpha = my.skill1;
}


// ---------------------------------------------------------------------
// View effects
VIEW fxa_mirror_view { };

// action: FXA_Mirror
// title: Mirror
// Desc: Create a floor mirror.
// Code to simulate a floor mirror by generating
// a vertically flipped camera view from below through the floor
action FXA_Mirror
{
// initialize camera portal for realtime reflections
ifdef CAPS_MIRROR;
	camera.portal = fxa_mirror_view;
	fxa_mirror_view.noshadow = on;	// suppress shadows in the mirror
	fxa_mirror_view.portalclip = on;	// clip at portal plane
	while(1)
	{
		proc_late();	// place it at the end of the function list - the camera must be moved before
		fxa_mirror_view.genius = camera.genius;
		fxa_mirror_view.aspect = -camera.aspect;	// flip the image upside down
		fxa_mirror_view.arc 	  = camera.arc;
		fxa_mirror_view.fog 	  = camera.fog;
		fxa_mirror_view.fog_start = camera.fog_start;
		fxa_mirror_view.fog_end   = camera.fog_end;
		fxa_mirror_view.x 	= camera.x;
		fxa_mirror_view.y 	= camera.y;
		fxa_mirror_view.z 	= 2*camera.portal_z-camera.z;	// move the camera at its mirror position
		fxa_mirror_view.pan  = camera.pan;
		fxa_mirror_view.tilt = -camera.tilt;	// flip the vertical camera angle
		fxa_mirror_view.roll = -camera.roll;
		wait(1);
	}
endif;
	return;
}


endif;	// fxa01_wdl