ifndef invent_wdl;
define invent_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Inventory (01)
// class: INTERFACE
// type: USER
// image: invent01.pcx
// help: Handle a visual inventory.
//
// needs: gid01.wdl
// needs: trigger00.wdl
// needs: plSelect.wdl, cameraTarget.wdl
// needs: miscInput01.wdl, plBipedInput01.wdl
// needs: bipedPhy01.wdl,
// needs: bipedAnim01.wdl, bipedSnd01.wdl
// needs: plBiped01.wdl
// needs: plBipedHUD01.wdl
//
// prefix: invent01_
// idcode: 02C
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// section: Background Panel

// entry: Screen Pos
// cntl: spin -1 4 1
// entry_help: -1-off, 0-free, 1-upper left, 2-upper right, 3-lower left, 4-lower right
// id: 1
define	invent01_back_pos = 4;

// entry: Panel Bitmap
// id: 2
bmap invent01_backgrnd_bmp = <invent.pcx>;


// entry: X offset
// entry_help: Distance horz from edge
// id: 3
define	invent01_hposx = 0;
// entry: Y offset
// entry_help: Distance vert from edge
// id: 4
define	invent01_hposy = 0;


panel invent01_back_pan
{
	bmap = invent01_backgrnd_bmp;
	pos_X = plBipedHUD01_hposx;
	pos_y = plBipedHUD01_hposy;
	flags = overlay,refresh,visible;
}






// ---------------------------------------------------------------------
// section: Items

//TODO: Two items to start with (expand to six after testing)

// ---------------------------------------------------------------------
// section: Inventory Item #1

// entry: Item trigger ID (-1: off)
// help: the trigger number that activates this item icon
// cntl: spin -1 x 1
// id: 9
var	invent01_item1_id = -1;

// entry: Item Bitmap
// id: 10
bmap invent01_item1_bmp = <icon1.pcx>;

// entry: X offset
// help: Distance horz from edge
// id: 11
define	invent01_item1_hposx = 0;
// entry: Y offset
// help: Distance vert from edge
// id: 12
define	invent01_item1_hposy = 0;

panel invent01_item1_pan
{
	bmap = invent01_item1_bmp;
	pos_X = invent01_item1_hposx;
	pos_y = invent01_item1_hposy;
	flags = overlay,refresh;
}

// ---------------------------------------------------------------------
// section: Inventory Item #2

// entry: Item trigger ID (-1: off)
// help: the trigger number that activates this item icon
// cntl: spin -1 x 1
// id: 14
var	invent01_item2_id = -1;

// entry: Item Bitmap
// id: 15
bmap invent01_item2_bmp = <icon1.pcx>;

// entry: X offset
// entry_help: Distance horz from edge
// id: 16
define	invent01_item2_hposx = 0;
// entry: Y offset
// entry_help: Distance vert from edge
// id: 17
define	invent01_item2_hposy = 0;

panel invent01_item2_pan
{
	bmap = invent01_item2_bmp;
	pos_X = invent01_item2_hposx;
	pos_y = invent01_item2_hposy;
	flags = overlay,refresh;
}


// ----------------------------------------------------------------------
// Starter


function	Invent01_Item_Pos_Update()
{
	if(invent01_back_pos > 0)
	{
		// use inventory background pos as a offset
   	invent01_item1_pan.pos_x = invent01_back_pan.pos_x + invent01_item1_hposx;
  		invent01_item1_pan.pos_y = invent01_back_pan.pos_y + invent01_item1_hposy;
   	invent01_item2_pan.pos_x = invent01_back_pan.pos_x + invent01_item2_hposx;
  		invent01_item2_pan.pos_y = invent01_back_pan.pos_y + invent01_item2_hposy;
	}

}

// Desc: Check to see if any icons need to be turned on or off
starter Invent01_Item_Check()
{
	while(1)
	{
		if(Trigger00_Check(invent01_item1_id,trigger00_bitlist) == 1)
		{
			invent01_item1_pan.visible = on;
		}
		else
		{
			invent01_item1_pan.visible = off;
		}
		if(Trigger00_Check(invent01_item2_id,trigger00_bitlist) == 1)
		{
			invent01_item2_pan.visible = on;
		}
		else
		{
			invent01_item2_pan.visible = off;
		}
		wait(5);
	}
}




// Desc: Create and update the background panel's position
starter	Invent01_Background()
{
	// used to store the old resolution
	var	invent01_old_back_mode = 0;

	// remove panel
	if(invent01_back_pos<0)
	{
		invent01_back_pan.visible = off;
		invent01_back_pan.refresh = off;
		return;
	}

	// panel uses set position
	if((invent01_back_pos == 0) || (invent01_back_pos > 4))
	{
		Invent01_Item_Pos_Update(); // update item positions
		return;
	}

	while(1)
	{
		if(invent01_old_back_mode !=  video_mode)
		{

			if(invent01_back_pos < 2)
			{
				// 1-upper left
				invent01_back_pan.pos_x = invent01_hposx;
				invent01_back_pan.pos_y = invent01_hposy;
			}
			else
			{
				if(invent01_back_pos < 3)
				{
					// 2-upper right,
					invent01_back_pan.pos_x = (screen_size.x - bmap_width(invent01_backgrnd_bmp)) - invent01_hposx;
					invent01_back_pan.pos_y = invent01_hposy;

				}
				else
				{
					if(invent01_back_pos < 4)
					{
						// 3-lower left
						invent01_back_pan.pos_x = invent01_hposx;
						invent01_back_pan.pos_y = (screen_size.y - bmap_height(invent01_backgrnd_bmp)) - invent01_hposy;

					}
					else
					{
						// 4-lower right
	 					invent01_back_pan.pos_x = (screen_size.x - bmap_width(invent01_backgrnd_bmp)) - invent01_hposx;
						invent01_back_pan.pos_y = (screen_size.y - bmap_height(invent01_backgrnd_bmp)) - invent01_hposy;
					}

				}
			}
			invent01_old_back_mode = video_mode;
			Invent01_Item_Pos_Update(); // update item positions
		}
		wait(5); 	// no need to update every frame
	}
}




/*
// Desc: Add an item to the inventory panel with the given id
//
// Return: 1 if successful, otherwise a negitive value (-1: full)
function	Invent01_Add_Item(id)
{

}

// Desc: Remove an item from the inventory with the given id
//
// Return: 1 if successful, otherwise a negitive value (-1: item not found)
function	Invent01_Remove_Item(id)
{

}
 */


endif;