ifndef lensFlare_wdl;
define lensFlare_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Lens Flare
// class: ENVIR
// type: USER
// image: lensFlare.pcx
// help: Create an 8 element "lens flare" in the level.
//
// prefix: lensFlare00_
// idcode: 00D
//
// ENDHEADER
// ---------------------------------------------------------------------


// enable: Start with lens flare active?
// id: 1
define lensFlare00_start_on_b;

//----------------------------------------------------------------------
// section: Sky texture identifier

// entry: String containing the name of the "sky" texture
// entry_help: If any part of the texture name scanned matches this, it is
// entry_help: concidered to be "sky" when calculating lens flare.
// id: 2
string	lensFlare00_skytex_str = "sky";

//----------------------------------------------------------------------
// section: Flare maps

// use the skill1 parameter to store a number in a sprite entity
// The pivot distance is the percent distance between the screen center
//(where pivot_dist = 0) and the sun (pivot_dist = 1).
define lensFlare00_pivot_dist,skill1;

// this is the sun itself
entity lensFlare00_sun_ent
{
// entry: Source (Sun) flare pict
// id: 3
	type = <flare2.pcx>; // uses a sprite
	view = CAMERA; 		// same camera parameters as the default view
	layer = -6; 			// displayed beneath other entity layers
// entry: Pivot Dist
// id: 4
	lensFlare00_pivot_dist = 1;		// distance factor from 'pivot point', must be 1 for sun
// entry: Scale in X
// id: 5
	scale_x = 2;
// entry: Scale in Y
// id: 6
	scale_y = 2;
}

// The 8 lens flare entities
entity lensFlare00_0_ent
{
// entry: Flare pict 0
// id: 7
	type = <flare0.pcx>;
	view = CAMERA;
	layer = -6;
// entry: Pivot Dist
// id: 8
	lensFlare00_pivot_dist = 0.75;		// distance factor from 'pivot point', must be 1 for sun
// entry: Scale in X
// id: 9
	scale_x = 1;
// entry: Scale in Y
// id: 10
	scale_y = 1;
}
entity lensFlare00_1_ent
{	// 7 lens reflections
// entry: Flare pict 1
// id: 11
	type = <flare1.pcx>;
	view = CAMERA;
	layer = -6;
// entry: Pivot Dist
// id: 12
	lensFlare00_pivot_dist = 0.55;
// entry: Scale in X
// id: 13
	scale_x = 1;
// entry: Scale in Y
// id: 14
	scale_y = 1;
}
entity lensFlare00_2_ent
{
// entry: Flare pict 2
// id: 15
 	type = <flare2.pcx>;
	view = CAMERA;
	layer = -6;
// entry: Pivot Dist
// id: 16
	lensFlare00_pivot_dist = 0.35;
// entry: Scale in X
// id: 17
	scale_x = 1;
// entry: Scale in Y
// id: 18
	scale_y = 1;
}

entity lensFlare00_3_ent
{
// entry: Flare pict 3
// id: 19
	type = <flare3.pcx>;
	layer = -6;
	view = CAMERA;
// entry: Pivot Dist
// id: 20
	lensFlare00_pivot_dist = 0.15;
// entry: Scale in X
// id: 21
	scale_x = 1.5;
// entry: Scale in Y
// id: 22
	scale_y = 1.5;
}

entity lensFlare00_4_ent
{
// entry: Flare pict 4
// id: 23
 	type = <flare0.pcx>;
	layer = -6;
	view = CAMERA;
// entry: Pivot Dist
// id: 24
	lensFlare00_pivot_dist = -0.25;
// entry: Scale in X
// id: 25
	scale_x = 1;
// entry: Scale in Y
// id: 26
	scale_y = 1;
}

entity lensFlare00_5_ent
{
// entry: Flare pict 5
// id: 27
 	type = <flare1.pcx>;
	layer = -6;
	view = CAMERA;
// entry: Pivot Dist
// id: 28
	lensFlare00_pivot_dist = -0.45;
// entry: Scale in X
// id: 29
	scale_x = 1;
// entry: Scale in Y
// id: 30
	scale_y = 1;
}

entity lensFlare00_6_ent
{
// entry: Flare pict 6
// id: 31
	type = <flare2.pcx>;
	layer = -6;
	view = CAMERA;
// entry: Pivot Dist
// id: 32
	lensFlare00_pivot_dist = -0.65;
// entry: Scale in X
// id: 33
	scale_x = 1;
// entry: Scale in Y
// id: 34
	scale_y = 1;
}

entity lensFlare00_7_ent
{
// entry: Flare pict 7
// id: 35
	type = <flare3.pcx>;
	layer = -6;
	view = CAMERA;
// entry: Pivot Dist
// id: 36
	lensFlare00_pivot_dist = -0.85;
// entry: Scale in X
// id: 37
	scale_x = 1.5;
// entry: Scale in Y
// id: 38
	scale_y = 1.5;
}


//------------------------------------------------------------------
// Local vars.
var lensFlare00_trace_mode; // used to trace to the sun

var lensFlare00_valid = -1;	// -1 == not created
	 				    			// 0 == off
	 					 			// 1 == on
	  					 			// otherwise == turning off



// Desc: this function takes an entity as parameter.
function LensFlare00_Init(lensFlare00_ent)
{
	my = lensFlare00_ent;	// necessary because function parameters have no type
	my.visible = off;	// start with flares off
	if (video_depth > 8) 	// D3D mode?
	{
		ent_alphaset(0,10);	// create an alpha channel (won't work with standard edition)
		my.bright = on;
		my.flare = on;
	}
	else
	{
		my.transparent = on;	// looks bad in 8 bit
	}
}


// Desc: places a flare at temp.x/temp.y deviations from screen center
function LensFlare00_Place(lensFlare00_ent)
{
	my = lensFlare00_ent;
	my.visible = on;

	// multiply the pixel deviation with the pivot factor,
	// and add the screen center
	my.x = temp.x*my.lensFlare00_pivot_dist + 0.5*screen_size.x;
	my.y = temp.y*my.lensFlare00_pivot_dist + 0.5*screen_size.y;
	my.z = 750;	// screen distance, determines the size of the flare
	rel_for_screen(my.x,camera);
}


// Desc: this function turns all the lensFlare00_N_ent (and lensFlare00_sun_ent) on or
// 	off depending on the value pass in 'on_off'.
//
function LensFlare00_Visible(on_off)
{
	lensFlare00_sun_ent.visible = on_off;

	lensFlare00_0_ent.visible = on_off;
	lensFlare00_1_ent.visible = on_off;
	lensFlare00_2_ent.visible = on_off;
	lensFlare00_3_ent.visible = on_off;
	lensFlare00_4_ent.visible = on_off;
	lensFlare00_5_ent.visible = on_off;
	lensFlare00_6_ent.visible = on_off;
	lensFlare00_7_ent.visible = on_off;
}



// Desc: setup the lens flare effect
function LensFlare00_Create()
{
	// set alpha values for each entity
	LensFlare00_Init(lensFlare00_0_ent);
	LensFlare00_Init(lensFlare00_1_ent);
	LensFlare00_Init(lensFlare00_2_ent);
	LensFlare00_Init(lensFlare00_3_ent);
	LensFlare00_Init(lensFlare00_4_ent);
	LensFlare00_Init(lensFlare00_5_ent);
	LensFlare00_Init(lensFlare00_6_ent);
	LensFlare00_Init(lensFlare00_7_ent);

	LensFlare00_Init(lensFlare00_sun_ent);	// <<the sun lensFlare00_

	//  set the trace mode to be used to trace to the sun
	lensFlare00_trace_mode = IGNORE_PASSABLE + IGNORE_MODELS; //IGNORE_PASSABLE needed for tracing through the sky box)

	lensFlare00_Valid = 0;	// start 'off'
}


// Desc: start and animate a lens flare effect as long as qLensFlare == 1
//
starter	LensFlare00_Start()
{
	diag("\nWDL-Loaded:lensFlare.wdl");

	ifndef lensFlare00_start_on_b;
		return; // don't init lensFlare yet
	endif;

	if(lensFlare00_valid == -1) // create the lens flares
	{
		LensFlare00_Create();
	}

	if(lensFlare00_valid == 1)  // lens flare already started
	{
		return;
	}

	// allow for setup time for "lensflare_create"
	//(if it was called right before this function)
	wait(1);
	lensFlare00_valid = 1;	// mark lens flare as on

	// wait until the sun_pos vector is valid
	while((sun_pos.x == 0) && (sun_pos.y == 0) && (sun_pos.z == 0))
	{ wait(1); }

	// place lens flares
	while(lensFlare00_valid == 1)
	{
		wait(1);	// animate each cycle
		// Animate lens flare
		vec_set(temp,sun_pos);
		if(vec_to_screen(temp,camera) == 0)
		{
		// Outside of View cone... remove lens flares
			LensFlare00_Visible(off);
		}
		else     // check for trace to sun
		{
			// trace to the 'sun'
			trace_mode = lensFlare00_trace_mode + scan_texture;
			// check if line to sun is blocked
			c_trace(camera.x,sun_pos,trace_mode);
			if(trace_hit == 1)
			{
				if( str_stri(tex_name,lensFlare00_skytex_str) == 0) // object blocking us is not sky
				{
					// Something is blocking us.. hide lens flare
					LensFlare00_Visible(off);
					continue;	// continue loop
				}
			}


			// temp now contains the sun XY screen position
			// subtract the screen center, needed for flare_place()
			temp.x -= 0.5 * screen_size.x;
			temp.y -= 0.5 * screen_size.y;
			LensFlare00_Place(lensFlare00_sun_ent);

			// place all lensFlare00_s according to deviation and their pivot distance
			LensFlare00_Place(lensFlare00_0_ent);
			LensFlare00_Place(lensFlare00_1_ent);
			LensFlare00_Place(lensFlare00_2_ent);
			LensFlare00_Place(lensFlare00_3_ent);
			LensFlare00_Place(lensFlare00_4_ent);
			LensFlare00_Place(lensFlare00_5_ent);
			LensFlare00_Place(lensFlare00_6_ent);
			LensFlare00_Place(lensFlare00_7_ent);
		}
	}

	// Remove lens flares
	LensFlare00_Visible(off);
	lensFlare00_Valid = 0;     // mark lens flare as off
}

// Desc: stop the lens flare effect
function LensFlare00_Stop()
{
	lensFlare00_valid = .5;	// signal to stop
}

endif;