ifndef material_wdl;
define material_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Materials
// class: ENVIR
// type: USER
// image: testimage.pcx
// help: C-Script for tuning materials
//
// Dev Notes: (c) JCL / Conitec 2003
//
// idcode: 032
//
// ENDHEADER
// ---------------------------------------------------------------------



////////////////////////////////////////////////////////////
// helper functions for realtime material adjustment
var _mat_type = 0;
var _mat_prop = 0;
var _mat_typemax = 6;
var _mat_propmax = 6;
var _mat_value;

material* _mat;
string _matset_str[40];

text _mattype_txt {
	strings 6;
	string = "mat_shaded","mat_flat","mat_terrain",
		"mat_model","mat_sprite","mat_metal";
}

text _matprop_txt {
	strings 6;
	string = "ambient","diffuse","specular","power","albedo","emissive";
}

text _matset_txt {
	string = _matset_str;
}

function _mat_set(&matcolor,color)
{
	color = clamp(color,0,255);
	vec_set(matcolor,vector(color,color,color));
}

function _mat_setprop()
{
	if (_mat_prop == 0) { _mat_set(_mat.ambient_blue,_mat_value); }
	if (_mat_prop == 1) { _mat_set(_mat.diffuse_blue,_mat_value); }
	if (_mat_prop == 2) { _mat_set(_mat.specular_blue,_mat_value); }
	if (_mat_prop == 3) { _mat.power = _mat_value; }
	if (_mat_prop == 4) { _mat.albedo = _mat_value; }
	if (_mat_prop == 5) { _mat_set(_mat.emissive_blue,_mat_value); }
}

function _mat_display()
{
	_mat_type = cycle(_mat_type,0,_mat_typemax);
	_mat_prop = cycle(_mat_prop,0,_mat_propmax);

	if (_mat_type == 0) { _mat = mat_shaded; }
	if (_mat_type == 1) { _mat = mat_flat; }
	if (_mat_type == 2) { _mat = mat_terrain; }
	if (_mat_type == 3) { _mat = mat_model; }
	if (_mat_type == 4) { _mat = mat_sprite; }
	if (_mat_type == 5) { _mat = mat_metal; }

	if (_mat_prop == 0) { _mat_value = _mat.ambient_blue; }
	if (_mat_prop == 1) { _mat_value = _mat.diffuse_blue; }
	if (_mat_prop == 2) { _mat_value = _mat.specular_blue; }
	if (_mat_prop == 3) { _mat_value = _mat.power; }
	if (_mat_prop == 4) { _mat_value = _mat.albedo; }
	if (_mat_prop == 5) { _mat_value = _mat.emissive_blue; }

	_matset_txt.pos_x = 2;
	_matset_txt.pos_y = screen_size.y - 12;
	_matset_txt.visible = on;

	str_cpy(_matset_txt.string,_mattype_txt.string[_mat_type]);
	str_cat(_matset_txt.string,".");
	str_cat(_matset_txt.string,_matprop_txt.string[_mat_prop]);
	str_cat(_matset_txt.string," = ");
	str_cat(_matset_txt.string,str_for_num(_tstr,_mat_value));
}

function _mat_adjust()
{
	proc_kill(4);
	freeze_mode = 1;

	_mat_type += 1;

	while(1)
	{
		if (key_esc) {
			_matset_txt.visible = off;
			freeze_mode = 0;
			return;
		}

		if (key_cuu) {
			_mat_prop += 1;
			_mat_display();
			sleep(0.2);
		}

		if (key_cud) {
			_mat_prop -= 1;
			_mat_display();
			sleep(0.2);
		}

		if (key_cur) {
			if (_mat_prop == 3) {
				_mat_value += 0.5; }
			else {
				_mat_value += 5; }
			_mat_setprop();
			_mat_display();
			sleep(0.2);
		}

		if (key_cul) {
			if (_mat_prop == 3) {
				_mat_value -= 0.5; }
			else {
				_mat_value -= 5; }
			_mat_setprop();
			_mat_display();
			sleep(0.2);
		}
		_mat_display();
		wait(1);
	}
}

//////////////////////////////////////////////////////////////////////
//title: Material Setup


starter _mat_setup
{
	var color;

//enable: Enable realtime material adjustment
//help: If selected, material can be adjusted in real time.
//help: During runtime, press [M] for adjusting default materials.
//help: The selected material and property appears below on the screen.
//help: [M] toggles the default material to adjust.
//help: Cursor up / down select different material properties.
//help: Cursor left / right change the value.
//help: [Esc] quits.
	on_m = _mat_adjust;

//enable: Disable custom material adjustment
//help: Set this flag to use the default values
//help: (ignoring what is set below).
//	return;

//----------------------------------------------------------------------
//section: Material for shaded blocks (mat_shaded)
	_mat = mat_shaded;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 255;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 255;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 0;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 0;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 0;

//----------------------------------------------------------------------
//section: Material for flat blocks (mat_flat)
	_mat = mat_flat;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 255;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 0;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 0;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 0;

//----------------------------------------------------------------------
//section: Material for terrain (mat_terrain)
	_mat = mat_terrain;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 200;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 255;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 0;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 0;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 50;

//----------------------------------------------------------------------
//section: Material for models (mat_model)
	_mat = mat_model;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 200;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 255;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 0;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 0;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 50;

//----------------------------------------------------------------------
//section: Material for sprites (mat_sprite)
	_mat = mat_sprite;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 200;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 128;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 0;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 0;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 0;

//----------------------------------------------------------------------
//section: Material for metal (mat_metal)
	_mat = mat_metal;

//entry: Emissive (0..255)
//help: Adjust the emissive value to set the brightness of the material
//help: regardeless of it's environment.
// cntl: spin 0 255 1
	color = 0;
	_mat_set(_mat.emissive_blue,color);

//entry: Ambient (0..255)
//help: Adjust the ambient value to set the influence of
//help: static lights and entity and camera ambient.
// cntl: spin 0 255 1
	color = 100;
	_mat_set(_mat.ambient_blue,color);

//entry: Diffuse (0..255)
//help: Adjust this value to set the diffuse reflection
//help: of the sun or dynamic lights.
// cntl: spin 0 255 1
	color = 100;
	_mat_set(_mat.diffuse_blue,color);

//entry: Specular (0..255)
//help: Adjust this value to set the specular reflection
//help: of the sun or dynamic lights (also influenced by Power).
//help: This can be used to create a metal or plastic look.
// cntl: spin 0 255 1
	color = 200;
	vec_set(_mat.specular_blue,color);

//entry: Power (0..10)
//help: Adjust this value to set 'sharpness for specular reflections.
//help: Higher values give the material a 'plastic' look.
// cntl: spin 0 10 1
	_mat.power = 4;

//entry: Albedo (0..100)
//help: Adjust this value to set the reflection of sunlight
//help: depending on the entities' static environment light.
//help: Low values disable any effect of sun light,
//help: high values enable sun light even in dark places.
// cntl: spin 0 100 1
	_mat.albedo = 50;

//----------------------------------------------------------------------
//section: Material for dynamic shadows (mat_shadow)
	_mat = mat_shadow;

//entry: Alpha (0..100)
//help: Adjust this value to set the opacity of dynamic shadows.
// cntl: spin 0 100 1
	_mat.alpha = 50;
}

endif;