ifndef preform_wdl;
define preform_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Preformance Diag (01)
// class: TOOLS
// type: USER
// image: preform01.pcx
// help: Output game statistics for debugging.
//
//
// needs: gid01.wdl
//
// prefix: preform01_
// idcode: 02D
//
// ENDHEADER
// ---------------------------------------------------------------------


var_nsave	perform01_outfile;

// entry: Polling Frames
// entry_help: Number of frames between polls (1-once per frame, 2-once every second frame, etc.)
// id: 1
define	perform_poll_wait = 60;


var	perform01_total_time;   // total time running
var 	perform01_max_time;		// peak frame (lowest FPS)
var 	perform01_min_time;		// quickest frame (highest FPS)
var	perform01_avg_time;		// average time

function	Perform01_Timing(outfile)
{

	file_str_write(outfile,"\n\nTIMING\n======");
	// TIMING
	file_str_write(outfile,"\ntotal_ticks: ");
	file_var_write(outfile,total_ticks);
	file_str_write(outfile,"\ntotal_secs: ");
	file_var_write(outfile,total_secs);

	file_str_write(outfile,"\n\ncurrent tick: ");
	file_var_write(outfile,time);
	file_str_write(outfile,"\n\nAprox FPS: ");
	if(time != 0)
	{
		file_var_write(outfile,(16/time));
	}
	else
	{
		file_var_write(outfile,999999);
	}
	file_str_write(outfile,"\ntime_smooth?: ");
	file_var_write(outfile,time_smooth);
  	file_str_write(outfile,"\ntime_factor?: ");
	file_var_write(outfile,time_factor);
	file_str_write(outfile,"\nPhysic tick: ");
	file_var_write(outfile,time_physics);
}


function	Perform01_TimeFor(outfile)
{
	var	total_ms;
	var	render_ms;
	var	script_ms;

	total_ms  = max(0.001,(time*62.5));  // (time/16) * 1000
	render_ms = max(0.001,(time_level+time_sky+time_entities+time_effects+time_portals+time_panels));
	script_ms = max(0.001,(time_actions+time_pfuncs));

	// calc overall stats (used in Sum Report)
	perform01_total_time += time*perform_poll_wait;
	if(time > perform01_max_time) { perform01_max_time = time; }
	if(time < perform01_min_time) { perform01_min_time = time; }


	file_str_write(outfile,"\n\nMS (PER)\n========");
	// ms
	file_str_write(outfile,"\nRENDER: ");
	file_var_write(outfile,render_ms);
	file_str_write(outfile," (");
	file_var_write(outfile,((render_ms/total_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n   Level: ");
	file_var_write(outfile,time_level);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_level/render_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n   Sky: ");
	file_var_write(outfile,time_sky);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_sky/render_ms) * 100));
	file_str_write(outfile,"%)");

  	file_str_write(outfile,"\n   Level Entities: ");
	file_var_write(outfile, time_entities);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_entities/render_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n   Particles: ");
	file_var_write(outfile, time_effects);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_effects/render_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n   Portals & Mirrors: ");
	file_var_write(outfile, time_portals);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_portals/render_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n   Panels and Screen Ents: ");
	file_var_write(outfile, time_panels);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_panels/render_ms) * 100));
	file_str_write(outfile,"%)");


	file_str_write(outfile,"\n\nScreen Refresh: ");
	file_var_write(outfile, time_update);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_update/total_ms) * 100));
	file_str_write(outfile,"%)");

	file_str_write(outfile,"\n\nScript execution: ");
	file_var_write(outfile, script_ms);
	file_str_write(outfile," (");
	file_var_write(outfile,((script_ms/total_ms) * 100));
	file_str_write(outfile,"%)");
	file_str_write(outfile,"\n   actions: ");
	file_var_write(outfile,time_actions);
	file_str_write(outfile,"\n   particles: ");
	file_var_write(outfile,time_pfuncs);


	file_str_write(outfile,"\n\nPhysics: ");
	file_var_write(outfile, time_physics);
	file_str_write(outfile," (");
	file_var_write(outfile,((time_physics/total_ms) * 100));
	file_str_write(outfile,"%)");
}

function	Perform01_Camera(outfile)
{
		// Location of Camera
		file_str_write(outfile,"\n\nCAMERA\n======");
		file_str_write(outfile,"\nPos(x,y,z): ");
   	file_var_write(outfile,camera.x);
 		file_str_write(outfile,", ");
 		file_var_write(outfile,camera.y);
 		file_str_write(outfile,", ");
  		file_var_write(outfile,camera.z);
		file_str_write(outfile,"\nAng(pan,tilt,roll): ");
   	file_var_write(outfile,camera.pan);
 		file_str_write(outfile,", ");
 		file_var_write(outfile,camera.tilt);
 		file_str_write(outfile,", ");
  		file_var_write(outfile,camera.roll);
}

function	Perform01_Count(outfile)
{
	file_str_write(outfile,"\n\nCOUNT\n=====");

		file_str_write(outfile,"\nLevel Entities: ");
		file_var_write(outfile,num_entities);
		file_str_write(outfile,"  (MAX: ");
		file_var_write(outfile,max_entities);
		file_str_write(outfile,")");
		file_str_write(outfile,"\nRunning Actions: ");
		file_var_write(outfile,num_actions);

		// Visible polys/entities
		file_str_write(outfile,"\nVISIBLE\n=======");
		file_str_write(outfile,"\nTriangles: ");
		file_var_write(outfile,num_vistriangles);
		file_str_write(outfile,"\nMap Polys: ");
		file_var_write(outfile,num_vismappolys);
		file_str_write(outfile,"\nEntities: ");
		file_var_write(outfile,num_visents);
		file_str_write(outfile,"\nEntity Polys: ");
		file_var_write(outfile,num_visentpolys);
		file_str_write(outfile,"\nParticles: ");
		file_var_write(outfile,num_particles);
		file_str_write(outfile,"  (MAX: ");
		file_var_write(outfile,max_particles);
		file_str_write(outfile,")");
}


// Desc: Run an sum of performance stats to this point
// Hint: Call this on exit
function	Perform01_ReportSum()
{
		file_str_write(perform01_outfile,"\n\n=================\n=================\nSUM REPORT\n=======");

		file_str_write(perform01_outfile,"\nTotal Time: ");
		file_var_write(perform01_outfile,perform01_total_time);

		file_str_write(perform01_outfile,"\nMax Time: ");
		file_var_write(perform01_outfile,perform01_max_time);
		file_str_write(perform01_outfile," (Aprox FPS: ");
		if(perform01_outfile != 0)
		{
			file_var_write(perform01_outfile,(16/perform01_max_time));
		}
		else
		{
			file_var_write(perform01_outfile,999999);
		}
		file_str_write(perform01_outfile,")");

		file_str_write(perform01_outfile,"\nMin Time: ");
		file_var_write(perform01_outfile,perform01_min_time);
		file_str_write(perform01_outfile," (Aprox FPS: ");
		if(perform01_outfile != 0)
		{
			file_var_write(perform01_outfile,(16/perform01_min_time));
		}
		else
		{
			file_var_write(perform01_outfile,999999);
		}
		file_str_write(perform01_outfile,")");


		perform01_avg_time = perform01_total_time/total_frames;
		file_str_write(perform01_outfile,"\nAvg Time: ");
		file_var_write(perform01_outfile,perform01_avg_time);
		file_str_write(perform01_outfile," (Aprox FPS: ");
		if(perform01_outfile != 0)
		{
			file_var_write(perform01_outfile,(16/perform01_avg_time));
		}
		else
		{
			file_var_write(perform01_outfile,999999);
		}
		file_str_write(perform01_outfile,")");

}




starter	Perform01()
{
	proc_late();

	// Open outfile
	perform01_outfile = file_open_write("perform_report.txt");
	if(perform01_outfile == 0)
	{
		diag("\nPerform01 :: Could not open outfile!!!\n");
		return;
	}

	// TIME STAMP
	file_str_write(perform01_outfile,"Engine Started (dd/mm/yyyy - hh:mm:ss): ");
	file_var_write(perform01_outfile,sys_day);
	file_str_write(perform01_outfile,"/");
	file_var_write(perform01_outfile,sys_month);
	file_str_write(perform01_outfile,"/");
	file_var_write(perform01_outfile,sys_year);
	file_str_write(perform01_outfile," - ");
	file_var_write(perform01_outfile,sys_hours);
	file_str_write(perform01_outfile,":");
	file_var_write(perform01_outfile,sys_minutes);
	file_str_write(perform01_outfile,":");
	file_var_write(perform01_outfile,sys_seconds);


	// Engine info
	file_str_write(perform01_outfile,"\nVersion: ");
	file_var_write(perform01_outfile,version);
	file_str_write(perform01_outfile,"\nEdition: ");
	file_var_write(perform01_outfile,edition);



	// Video output
	file_str_write(perform01_outfile,"\nVIDEO\n=====");
	file_str_write(perform01_outfile,"\nvideo_mode: ");
	file_var_write(perform01_outfile,video_mode);
	file_str_write(perform01_outfile,"\nvideo_depth: ");
	file_var_write(perform01_outfile,video_depth);
	file_str_write(perform01_outfile,"\nvideo_screen: ");
	file_var_write(perform01_outfile,video_screen);

	// FPS
	file_str_write(perform01_outfile,"\nFPS\n===");
	file_str_write(perform01_outfile,"\nmin: ");
	file_var_write(perform01_outfile,fps_min);
	file_str_write(perform01_outfile,"\nmax: ");
	file_var_write(perform01_outfile,fps_max);
	file_str_write(perform01_outfile,"\nlock: ");
	file_var_write(perform01_outfile,fps_lock);

	// Misc
	file_str_write(perform01_outfile,"\n\nwarn_level: ");
	file_var_write(perform01_outfile,warn_level);
	file_str_write(perform01_outfile,"\ntex_share: ");
	file_var_write(perform01_outfile,tex_share);
	file_str_write(perform01_outfile,"\nJoystics: ");
	file_var_write(perform01_outfile,num_joysticks);

// GAME START

	// Reset values
	perform01_total_time = 0;
	perform01_max_time = 0;
	perform01_min_time = 999999;
	perform01_avg_time = 0;

	while(1)
	{
		// PER FRAME
		file_str_write(perform01_outfile,"\n\n=======================================================");

		file_str_write(perform01_outfile,"\ntotal_frames: ");
		file_var_write(perform01_outfile,total_frames);

		// TIME...
 		Perform01_Timing(perform01_outfile);

		// TIME SPENT IN...
		Perform01_TimeFor(perform01_outfile);


		// Memory
/*  		file_str_write(perform01_outfile,"\n\nNexus: ");
		file_var_write(perform01_outfile,nexus);

		// Location of Camera
		Perform01_Camera(perform01_outfile);


		// Count..
		Perform01_Count(perform01_outfile);

		// MISC
		file_str_write(perform01_outfile,"\nMISC\n====");
		file_str_write(perform01_outfile,"\nMedia Streams: ");
		file_var_write(perform01_outfile,num_mediastreams);
*/

		wait(perform_poll_wait);	//


	}



}



endif;