ifndef plCar_wdl;
define plCar_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4.1
// date: 050615
//
// title: Car Player (01)
// class: PLAYER
// type: USER
// image: plCar01.pcx
// help: Player controlled car.
// help: Requires *PRO* edition to run.
//
// needs: gid01.wdl
// needs: plSelect.wdl, cameraTarget.wdl
// needs: miscInput01.wdl,
// needs: trigger00.wdl, particle00.wdl
//
// uses: plSelect::plSelect_curr_model,
//
// prefix: plCar01_
// idcode: 016
//
// ENDHEADER
// ---------------------------------------------------------------------


// Local Function Prototypes
function	PlCar01_Activate();	// activate the car model
function	PlCar01_Deactivate();	// deactivate the car model
function	PlCar01_Update();		// update mode (call once per frame while active)
function	PlCar01_Death();			// handle death
function PlCar01_EventExit();	// return control to the previous player entity


// entry: ID Number
// entry_help: Identification number for this player model (used in plSelect)
// entry_help: NOTE: Action not defined if two player models share a single value
// id: 1
define	plCar01_id = 2;

// pointer to the player car entity
entity*	plCar01_entity;





////////////////////////////////////////////////////////////////////////
// Car Skill Defines
define	_health__016,SKILL1;	// "health" of the vehicle

// Physical Car constants Skill Defines
define _maxTorque__016,SKILL2; //= 900; // maximum torque applied to rear wheels
define _maxTorqueBrake__016,SKILL3;//=1000; 	// this will be applied to rear wheels when hitting brakes or handbrake
define _maxAngSpeed__016,SKILL4;//= 60; //70  		// max. angular speed of wheels; max linear speed depends on wheel radius
define _massWheel__016,SKILL5;//=		30;		// mass per wheel
define _massChassis__016,SKILL6;//=	10; 		// mass for chassis lighter than wheels so it doesn't tilt
define _fricWheel__016,SKILL7;//= 		100; //100	// friction of wheels -god for "in the water"
define _fricChassis__016,SKILL8;//=	20;		// friction of chassis (low to avoid getting stuck on ramps
define _dampChassis__016,SKILL9;//= 	5;		// damping lin/ang for chassis
define _dampWheel__016,SKILL10;//=		15; //15		// damping ang for wheels
define _suspensionERP__016,SKILL11;//=  95000; //90000;	// how quickly shocks are reduced to correct position (100,000 max, 0: never)
define _suspensionCFM__016,SKILL12;//=  250; //500	// how far shocks can overshoot (0: never, 100,000 max)
define _gravity__016,SKILL13;//=1800;	//2000;		// gravity along -Z axis



//////////////////////////////////////////////////////////////
// Steering constants



var plCar01_commonWheelHeight=-100000;


//////////////////////////////////////////////////////////////
// these values depend on MDL and WMP where car is located
var plCar01_vecUp[3]= 			0,0,1; 		// up axis, usually +Z
var plCar01_vecRight[3]= 		0,-1,0;		// chassis right axis


// Camera offsets
var plCar01_viewOffset[3]= -50,-15,30; // where camera will be located relative to chassis center (cockpit)
var plCar01_viewAngle[3] = 0,0,0;


// Player exit offsets
var plCar01_exitVec[3] = 0,105,20;


//////////////////////////////////////////////////////////////
// Controller setup

// section: Keyboard Input

// entry: Keyboard Accel
// help: acceleration multiplier for key control
define plCar01_accelKey = 2;

// entry: Keyboard Steer
// help: acceleration multiplier for steering key control
define plCar01_accelSteerKey = 3;

// entry: Move forward key
// id: 4
string plCar01_move_for_key_c = "W";
var	plCar01_move_for_scancode = 0;
// entry: Move backwards key
// id: 5
string plCar01_move_back_key_c = "S";
var	plCar01_move_back_scancode = 0;
// entry: Turn left key
// id: 6
string plCar01_turn_left_key_c = "A";
var	plCar01_turn_left_scancode = 0;
// entry: Turn right key
// id: 7
string plCar01_turn_right_key_c = "D";
var	plCar01_turn_right_scancode = 0;
// entry: Brake key
// id: 8
string plCar01_brake_key_c = "space";
var	plCar01_brake_scancode = 0;
// entry: Horn key
// id: 9
string plCar01_horn_key_c = "Q";
var	plCar01_horn_scancode = 0;

// entry: Exit key
// id: 9
string plCar01_exit_key_c = "X";
var	plCar01_exit_scancode = 0;

//-----------------------------------------
// Section: Joystick Input

// entry_drop: Joystick
// help: 0: No joystick input
// help: 1: Treat joystick input as "digital" (on or off)
// help: 2: Treat joystick input as "analog"
// cntl: drop_list 0 1 2
var	plCar01_qJoystick = 2;

// entry: Joystick Brake button
// id: 10
string plCar01_brake_but_c = "joy_2";
var	plCar01_brake_but_scancode = 0;

// entry: Joystick Horn button
// id: 11
string plCar01_horn_but_c = "joy_3";
var	plCar01_horn_but_scancode = 0;

// entry: Joystick Exit button
// id: 12
string plCar01_exit_but_c = "joy_3";
var	plCar01_exit_but_scancode = 0;

// entry: Joy "Dead" X-axis
// help: "Dead Zone" for the joystick in the X-axis
// id: 13
define	plCar01_joyNullX = 20;

// entry: Joy "Dead" Y-axis
// help: "Dead Zone" for the joystick in the Y-axis
// id: 14
define	plCar01_joyNullY = 20;


// Desc: takes the current values stored in the user defined keys and creates scancodes
function	plCar01_SetKeys()
{
	// use MiscInput01_Key_For_Str() to convert strings into scancodes

	// forward key
	str_cpy(miscInput01_temp_str,plCar01_move_for_key_c);
	plCar01_move_for_scancode = MiscInput01_Key_For_Str();
	// reverse key
	str_cpy(miscInput01_temp_str,plCar01_move_back_key_c);
	plCar01_move_back_scancode = MiscInput01_Key_For_Str();
	// turn left key
	str_cpy(miscInput01_temp_str,plCar01_turn_left_key_c);
	plCar01_turn_left_scancode = MiscInput01_Key_For_Str();
	// turn right key
	str_cpy(miscInput01_temp_str,plCar01_turn_right_key_c);
	plCar01_turn_right_scancode = MiscInput01_Key_For_Str();

	// brake key
	str_cpy(miscInput01_temp_str,plCar01_brake_key_c);
	plCar01_brake_scancode = MiscInput01_Key_For_Str();

	// horn key
	str_cpy(miscInput01_temp_str,plCar01_horn_key_c);
	plCar01_horn_scancode = MiscInput01_Key_For_Str();

	// "Exit" key
	str_cpy(miscInput01_temp_str,plCar01_exit_key_c);
	plCar01_exit_scancode = MiscInput01_Key_For_Str();


	// joystick controls

	// brake button
	str_cpy(miscInput01_temp_str,plCar01_brake_but_c);
	plCar01_brake_but_scancode = MiscInput01_Key_For_Str();

	// horn button
	str_cpy(miscInput01_temp_str,plCar01_horn_but_c);
	plCar01_horn_but_scancode = MiscInput01_Key_For_Str();

	// "Exit" button
	str_cpy(miscInput01_temp_str,plCar01_exit_but_c);
	plCar01_exit_but_scancode = MiscInput01_Key_For_Str();


}


//////////////////////////////////////////////////////////////
// physics global vars

define _linSpeed__016, SKILL50;		// car speed in XY
define _downSpeed__016, SKILL51;		// car speed in Z direction
define _angSpeed__016, SKILL52;		// current wheel angular speed along y axis of wheel
define _targetSpeed__016, SKILL53;	// desired angular speed (if torque permits)
define _targetSteer__016, SKILL54;  // desired steering angle
define _wheelFL__016, SKILL55;		// wheel constraint handles
define _wheelFR__016, SKILL56;
define _wheelRL__016, SKILL57;
define _wheelRR__016, SKILL58;


entity* plCar01_entity; // pointer to car object
var plCar01_wheelCounter=0; // # of initialized wheels

// pointer to tire objects
entity* plCar01_FL; entity* plCar01_FR;
entity* plCar01_RL; entity* plCar01_RR;







//////////////////////////////////////////////////////////////////////////////////////////
// Resources
sound plCar01_engine_wav = <engine.wav>; // player's engine sound
sound plCar01_horn_wav = <horn.wav>; // player's horn
sound plCar01_crash_wav = <crash.wav>; // player's crashing sound
sound plCar01_tires_wav = <tires.wav>; // tires squealing
sound plCar01_flap_wav = <lp_flap.wav>; // tires squealing


var_nsave plCar01_hTireSnd;			// handle to sound playing

bmap plCar01_Smoke_bmp = <smoke.tga>;

// ---------------------------------------------------------------------
// FUNCTIONS

starter PlCar01_Load()
{
	diag("\nWDL-Loaded:plCar01.wdl");


	plCar01_SetKeys();	// setup the input keys
}

//////////////////////////////////////////////////////////////////////////////////////////
// Particle and event function

///////////////////////////////////////////////////////////////////
// helper function: sets the vector to random direction   and length

// helper function: fades out a particle
function plCar01_part_alphafade()
{
   my.alpha -= time;
   my.size  += time;
   if (my.alpha <= 0) { my.lifespan = 0; } else { my.lifespan=5; }
}

// particle function: generates a fading explosion into vel direction
function plCar01_SmokeAction()
{
   Particle00_Randomize(temp,1.5);
   // extract passed values
   var speed; speed= my.vel_x;
   my.vel_x=0;
   vec_add (my.vel_x, temp);
   my.bmap = plCar01_Smoke_bmp;
   my.move = on;
   my.function = plCar01_part_alphafade; // change to a shorter, faster function

   if (random(2)>1)
   {
		 // every other particle is white
		my.bright = on;
   		my.alpha = 10 + random(10);
   		my.size= 1+speed*random(10);
   } else {
   		my.alpha = 5 + random(25);
   		my.size= 5+speed*random(30);
   }
}


//////////////////////////////////////////////////////////////////////////////////////////
//
function PlCar01_EventBrake(){
	if (!snd_playing(plCar01_hTireSnd))
	{
		plCar01_hTireSnd = snd_play (plCar01_tires_wav, 55, 0);
		while (snd_playing(plCar01_hTireSnd))
		{
			var wheelDir[3];
			vec_set(wheelDir, vector(-15,random(16)-8,-15));
			var speed; speed= (abs(my._angSpeed__016)/(my._maxAngSpeed__016));
			vec_rotate(wheelDir, plCar01_entity.pan);
			vec_set(temp, plCar01_RL.x); vec_add(temp, wheelDir);
			effect(plCar01_SmokeAction, 5, temp, vector(speed,0,0));
			vec_set(temp, plCar01_RR.x); vec_add(temp, wheelDir);
			effect(plCar01_SmokeAction, 5, temp, vector(speed,0,0));
			wait(random(4));
		}
	}
}



//////////////////////////////////////////////////////////////////////////////////////////
//
var_nsave plCar01_horn_handle;

function PlCar01_EventHorn()
{
	if(!snd_playing(plCar01_horn_handle))
	{
		plCar01_horn_handle = snd_play(plCar01_horn_wav, 100, 0);
   }
}

//////////////////////////////////////////////////////////////////////////////////////////
// check if we crashed at high speed, then disassemble car
var_nsave plCar01_falp_handle;




//////////////////////////////////////////////////////////////////////////////////////////
// Car control

function PlCar01_Input()
{
	var direction = 0;
	var usejoy=0;


		if((plCar01_qJoystick > 0) && (joy_raw.y<-plCar01_joyNullY || joy_raw.y>plCar01_joyNullY))
		{
			// treat joystick as absolute device which is more
			// like a keyboard than a joystick
			if(plCar01_qJoystick == 1)
			{
				usejoy=0; direction= -(joy_raw.y / 255);
			}
			else
			{
				usejoy=1; direction= -(joy_raw.y-plCar01_joyNullY)/(255-plCar01_joyNullY);
			}
		}
		else
		{
			usejoy=0;
			if(Key_Pressed(plCar01_move_for_scancode))
			{
  				direction += 1;	// forward force
			}
			if(Key_Pressed(plCar01_move_back_scancode))
			{
  				direction -= 1;	// forward force
			}
		}

		// if we drive in one direction and want to change direction, apply brakes
		// (my._angSpeed__016>0 checks if rear wheel is turning forward
		// direction<0 checks if user hits reverse key
		if ((my._angSpeed__016>5 && direction<0) || (my._angSpeed__016<-5 && direction>0))
		{
			my._targetSpeed__016=0; // brake
			// press brakes quarter down when rolling at high speed
			if (abs(my._angSpeed__016)>0.25*my._maxAngSpeed__016)
			{
				phcon_setmotor(my._wheelRL__016, nullvector, vector(my._angSpeed__016*0.5, 0.5*my._maxTorqueBrake__016,0), nullvector);
				phcon_setmotor(my._wheelRR__016, nullvector, vector(my._angSpeed__016*0.5, 0.5*my._maxTorqueBrake__016,0), nullvector);
				if(abs(my._angSpeed__016)>0.50*my._maxAngSpeed__016)
				{
					PlCar01_EventBrake(); // braking sound
				}
			}
			else
			{
				// we're already slow- apply max. units brake torque
				phcon_setmotor(my._wheelRL__016, nullvector, vector(0, my._maxTorqueBrake__016,0), nullvector);
				phcon_setmotor(my._wheelRR__016, nullvector, vector(0, my._maxTorqueBrake__016,0), nullvector);
			}
		}
		else
		{
			var oldSpeed;
			oldSpeed = my._targetSpeed__016;
			// regular driving
			if(usejoy)
			{
				my._targetSpeed__016 = my._maxAngSpeed__016*direction; //absolute value for stick
			}
			else
			{
				// speed up over time for keyboard (relative acceleration)
				my._targetSpeed__016 += direction*plCar01_accelKey*time;
			}

			var torque; torque=my._maxTorque__016;
			// damp down speed over time if no key pressed
			if (!usejoy && direction==0)
			 {
				my._targetSpeed__016 *=1-(time*0.02); torque=my._maxTorque__016/2;
				if (abs(my._linSpeed__016)<20) // switch off opposing torque so we can roll down hills
				{ torque=0; my._targetSpeed__016 *=0.5;}
			} else {
				// use less torque to slow down or else we'll spin
				if (usejoy && abs(my._targetSpeed__016)<abs(my._angSpeed__016))
				{
					torque=my._maxTorque__016/2;
					//targetSpeed *=0.5;
				}

			}
			// max. reverse is 1/4 of forward speed
			my._targetSpeed__016 = clamp(my._targetSpeed__016, -my._maxAngSpeed__016*0.25, my._maxAngSpeed__016);
			phcon_setmotor(my._wheelRL__016, nullvector, vector(my._targetSpeed__016, torque,0), nullvector);
			phcon_setmotor(my._wheelRR__016, nullvector, vector(my._targetSpeed__016, torque,0), nullvector);
		}
		if(Key_Pressed(plCar01_brake_scancode)||((plCar01_qJoystick > 0) && Key_Pressed(plCar01_brake_but_scancode))) // handbrake
		{
			phcon_setmotor(my._wheelRL__016, nullvector, vector(0, my._maxTorqueBrake__016,0), nullvector);
			phcon_setmotor(my._wheelRR__016, nullvector, vector(0, my._maxTorqueBrake__016,0), nullvector);
			if (abs(my._linSpeed__016)>20)
			{ PlCar01_EventBrake(); }// squeaky brakes
		}


  // Steering

	direction=0;	//delta change
		if((plCar01_qJoystick > 0) && (joy_raw.x<-plCar01_joyNullX || joy_raw.x>plCar01_joyNullX))
		{
			// treat gamepad as absolute device which is more
			// like a keyboard than a joystick
			if(plCar01_qJoystick == 1)
			{
				usejoy=0; direction= (joy_raw.x / 255);
				direction*= PlCar01_CalcSteeringScale();
			}
			else
			{
				// using joystick
				usejoy=1; direction= (joy_raw.x-plCar01_joyNullX) / (255-plCar01_joyNullX);
				direction*= PlCar01_CalcSteeringScale();
			}
		}
		else
		{
			usejoy=0;
 			if(Key_Pressed(plCar01_turn_right_scancode))
			{
  				direction += 1;	// forward force
			}
			if(Key_Pressed(plCar01_turn_left_scancode))
			{
  				direction -= 1;	// forward force
			}
			direction*= PlCar01_CalcSteeringScale();
		}

		// steering control
		if (0==usejoy)
		{
			my._targetSteer__016 += direction*plCar01_accelSteerKey*time;
		}
		else
		{
			my._targetSteer__016 = 30*direction;
		}
		my._targetSteer__016 = clamp(my._targetSteer__016,-30,30);

		if ((my._angSpeed__016 > my._maxAngSpeed__016*0.95) && abs(my._targetSteer__016)>7)
		{
			PlCar01_EventBrake(); // play squeaky tires
		}

		if (direction!=0)
		{
			phcon_setparams2(my._wheelFL__016, vector(my._targetSteer__016,my._targetSteer__016,0), nullvector,
									vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
			phcon_setparams2(my._wheelFR__016, vector(my._targetSteer__016,my._targetSteer__016,0), nullvector,
									vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
		}
		else
		{
			my._targetSteer__016*=1-(time*0.25);
			if (abs(my._targetSteer__016)<1) { my._targetSteer__016=0; } // force to 0
			phcon_setparams2(my._wheelFL__016, vector(my._targetSteer__016,my._targetSteer__016,0), nullvector,
									vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
			phcon_setparams2(my._wheelFR__016, vector(my._targetSteer__016,my._targetSteer__016,0), nullvector,
									vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
		}

		if(Key_Pressed(plCar01_horn_scancode) || ((plCar01_qJoystick > 0) && Key_Pressed(plCar01_horn_but_scancode)))
		{
			PlCar01_EventHorn(); //sound horn
		}
		if(Key_Pressed(plCar01_exit_scancode) || ((plCar01_qJoystick > 0) && Key_Pressed(plCar01_exit_but_scancode)))
		{
			PlCar01_EventExit(); // exit the car
		}
}


//////////////////////////////////////////////////////////////////////
// the faster we go the more sensitive the car gets to steering. to make it easier for user
// scale down his steering inputs the faster we go
function PlCar01_CalcSteeringScale()
{
	return (1-(0.75*(abs(my._angSpeed__016)/(my._maxAngSpeed__016))));
}



//////////////////////////////////////////////////////////////////////////////////////////
// Continuously calculate wheel rotational speed and store in global var
// Also update the engine noise
function PlCar01_UpdateSpeed()
{
	while(plCar01_wheelCounter<4) { wait(1); } //wait for wheels to be init'ed
	while(plCar01_entity == NULL) { wait(1); }

	var players_engine; //handle for soundloop
	players_engine = snd_loop(plCar01_engine_wav, 100,0); // play the engine sound in a loop
	snd_tune(players_engine, 80, 80, 0);  //80% of real frequency
	
	// check the rotation of the rear tires
	var	rotSign;
	if(plCar01_RL.pan != plCar01_RR.pan)
	{
		// assume that the wheels are twisted 180 deg from each other
		rotSign = -1;
	}
	else
	{
		rotSign = 1;
	}

 	//var angFL; var angFR;
	var angRL; var angRR;
	var vecLin[3];
	while(plCar01_entity != NULL)
	{
		// get linear speed and tune engine sound
		phent_getvelocity(plCar01_entity, vecLin, nullvector); //linear speed
		plCar01_entity._downSpeed__016 = vecLin.z * 0.091; vecLin.z=0; // store Z separately
		plCar01_entity._linSpeed__016 = vec_length(vecLin) * 0.091; // convert from quant/sec to kmh

		if(plCar01_entity._maxAngSpeed__016 > 0)
		{
			var freq; freq=300*abs(my._angSpeed__016)/plCar01_entity._maxAngSpeed__016;
			snd_tune(players_engine, 5, freq+80, 0);  //80% of real frequency
		}

		// grab angular velocity vector from rear wheels
		phent_getangvelocity(plCar01_RL, temp); angRL=temp.y;
		phent_getangvelocity(plCar01_RR, temp); angRR=temp.y;
		// store largest one
		if (abs(angRL)>abs(angRR))
		{ my._angSpeed__016= rotSign * angRL; } else { my._angSpeed__016=angRR; }

    	wait(1);
	}
}



define _old_entity_handle, SKILL60;
define _old_model, SKILL61;




// Desc: save the id of the activating player and switch to the car model
function	PlCar01_Activate()
{
 	while(plCar01_entity == NULL) { wait(1); }

	// set the player select current entity to this biped entity
	plSelect_curr_ent = plCar01_entity;


	// make sure they are visible
	plCar01_entity.invisible = off;

	// make this the active player model
	PlSelect_Switch(plCar01_id);
}


// Desc: deactivate the car model
function	PlCar01_Deactivate()
{
	// hide the player car
//	plCar01_entity.visible = off;
temp=temp;
	//++ToDo: make this optional
}

// Desc:
function	PlCar01_Death()
{
	temp=temp;
 /*
 	// only enter once
	// note: this works because the state is set after this call!
	if(plSelect_state == plSelect_state_dead) { return; }


	plBiped01_death_pan.scale_x = (screen_size.x/bmap_width(plBiped01_death_bmp));
	plBiped01_death_pan.scale_y = (screen_size.y/bmap_height(plBiped01_death_bmp));
	plBiped01_death_pan.pos_x = 0;
	plBiped01_death_pan.pos_y = 0;
	plBiped01_death_pan.visible = on;
 	Clib_Panel_Fade(plBiped01_death_pan,5,100,3);
	wait(1); 		// NOTE: We need this wait because  plSelect_state_dead is not set until after this function is called
	while(plSelect_state == plSelect_state_dead)
	{
		wait(1);
	}

	// we should reach here only if the player heath is > 0
	plBiped01_death_pan.visible = off;
*/
}



function PlCar01_EventExit()
{
	if(plCar01_entity == NULL)	{ return(-1); } // no car entity
	if(plCar01_entity._old_model <= 0)	{ return(-2); } // nothing to return to

	// set YOU to the car
	YOU = plCar01_entity;

	// set ME to the old entity
	ME = ptr_for_handle(plCar01_entity._old_entity_handle);


	vec_set(ME.x,YOU.x);

	// Try to move outside the car
	vec_set(temp,plCar01_exitVec);
	vec_rotate(temp,plCar01_entity.pan);
	move_mode = ignore_you + ignore_passable + ignore_passents + ignore_push + glide + use_box;
	c_move(ME,nullvector,temp,move_mode);
	if(in_solid == 1) { return(-3); }	// we couldn't exit the car

	PlCar01_Deactivate();

	// switch the current entity to the old entity
	PlSelect_Switch(plCar01_entity._old_model);
	plCar01_entity._old_model = 0;
}


// Desc: update mode (call once per frame while active)
function	PlCar01_Update()
{
	// this function modifies/reads the current player biped entity
	my = plCar01_entity;

	// Get player input
	PlCar01_Input();



	// Send forces/flags to biped01 update (which will handle physics, animation, and sound)
 //	BipedPhy01_Update(plBiped01_force_vec, plBiped01_ang_force_vec.pan);

	// Handle player specific results from biped update
/*	if(my._biped01_state2_mod == biped01_state_mod_damage_const)
	{
		// handle any damage from biped update (falling, crushing, etc)
		my._health__003 -= my._biped01_state2_mod_value; // take damage (bipass armor)
		my._biped01_state2_mod = biped01_state2_mod_none_const;	// reset flag
	}

	// Handle drowning damage
	if(my._biped01_state_mod == biped01_state_mod_drown_const)
	{
		// handle any damage from drowning (time based)
		my._health__003 -= bipedPhy01_drowning_damage_const*TIME;
	}

*/
	// Check flags

/*	// Activation scan?
	if(plBiped01_flag_array[0] & 1)
	{
		temp[0] = plBiped01_activate_scan_horz_width;
		temp[1] = plBiped01_activate_scan_vert_width;
		temp[2] = plBiped01_activate_scan_range;
		gid01_event_type = gid01_event_scan_activate;
		scan_entity(my.x,temp);
	}
*/
}




// Desc: Add car model to plSelect list and execute the main update loop.
function PlCar01_Loop()
{
	var	updated_b;	// has the model been update yet this frame?
	var	bit_check;	// bit-field check value

	// wait until we have a valid player biped...
	while(plCar01_entity == null)
	{
		wait(1);
	}

	diag("\nWDL-Loaded:plCar.wdl");

	// init bit-field value used to check to see if this model is active
	bit_check = 1<<plCar01_id;

	// increase the player select "max_model" value if this value fall outside the current range
	if(bit_check > plSelect_max_model)
	{ plSelect_max_model = bit_check; }

	// add this model type to the collection of valid model types in player select
	plSelect_valid_models |= bit_check;

	while(1)
	{
		// only act if we have a valid entity...
		if(plCar01_entity != null)
		{
			updated_b = 0;	// model hasn't been updated yet this frame

			// Do this every frame if we are active or not
			PlCar01_UpdateSpeed(); // update rear wheel speed and engine noise (continuous)

			// if we are the active model
			if(plCar01_id == plSelect_curr_model)
			{
				// Activate the car model
				PlCar01_Activate();

				// while this model is active...
				while(plCar01_id == plSelect_curr_model)
				{
					if(gid01_level_state == gid01_level_loaded)
					{
						if(plCar01_entity._health__016 > 0)
						{
 							// Update the car model
 							PlCar01_Update();
							plSelect_state = plSelect_state_active;	// player is active
						}
						else
						{
  							// handle death
							PlCar01_Death();
							plSelect_state = plSelect_state_dead;		// player is dead
						}

						// Update the camera target
						cameraTarget_ent = plCar01_entity;
						vec_set(cameraTarget_pos_vec,plCar01_viewOffset);
						vec_rotate(cameraTarget_pos_vec,plCar01_entity.pan);
						vec_add(cameraTarget_pos_vec,plCar01_entity.x);
						vec_set(cameraTarget_ang_vec,plCar01_entity.pan);
						vec_add(cameraTarget_ang_vec,plCar01_viewAngle);
					}
					else
					{
						diag("\nPlayer Car is waiting...");
					}


					wait(1);
				}

				// Deactivate the car model
				PlCar01_Deactivate();

			}
		}
		wait(1);	// every frame
	}

}




// Desc: Event handler
function PlCar01_Event()
{
	//++ handle damage from shooting and explosions


	//
	if((YOU == plSelect_curr_ent) && (plSelect_curr_model != plCar01_id))
	{
		if((event_type == event_friction) || (event_type == event_scan) || (event_type == event_impact))
		{
			// save the old entity / model
			plCar01_entity._old_entity_handle = handle(plSelect_curr_ent);
			plCar01_entity._old_model = plSelect_curr_model;

			// make us the active model
			plSelect_curr_model = plCar01_id;

			return;
		}
	}



//	if (!snd_playing(plCar01_falp_handle)) {plCar01_falp_handle=snd_play (plCar01_flap_wav, 60, 0);} //cg

/*
	if((my._linSpeed__016>CRASH_SPEED && abs(Normal.z)<0.1)) {// heads on into wall
		snd_play (plCar01_crash_wav, 100, 0);
		wait(1);
		phcon_remove( my._wheelFL__016 ); phcon_remove( my._wheelFR__016 );
		phcon_remove( my._wheelRL__016 ); phcon_remove( my._wheelRR__016 );
		phent_setdamping( plCar01_FL, 80,50); 	phent_setdamping( plCar01_FR, 80,50);
		phent_setdamping( plCar01_RL, 80,50); 	phent_setdamping( plCar01_RR, 80,50);
		phent_setdamping( plCar01_entity, 80,50);

		var timer= 50;
		while (timer>0) {
			timer-= 1;
			vec_set(temp, plCar01_entity.x); //vec_add(temp, wheelDir);
			effect(plCar01_SmokeAction, 10, temp, vector(1,0,0));
			sleep(0.1);
		}
	}
*/
}



//////////////////////////////////////////////////////////////
// setup all 4 wheels
function PlCar01_InitWheel()
{
	while(plCar01_entity == NULL) { wait(1); }

	if (plCar01_commonWheelHeight<=-100000) {
          plCar01_commonWheelHeight=my.z; // store z for other wheels to come
        } else {
          my.z=plCar01_commonWheelHeight; // Set all wheels to same height
        }
	phent_settype(my, PH_RIGID, PH_SPHERE);
	phent_setmass(my, plCar01_entity._massWheel__016, PH_SPHERE);
	phent_setgroup(my, 2);
	phent_setfriction(my, plCar01_entity._fricWheel__016);
	phent_setdamping(my, 15,plCar01_entity._dampWheel__016);
	phent_setelasticity(my, 0, 100); // bounciness
	plCar01_wheelCounter+=1;
}

//----------------------------------------------------------------------
// Wheel Actions

action PlCar01_WheelFL
{
	my.passable=on;
	while(!plCar01_entity) { wait(1);}
	plCar01_FL=my;
	PlCar01_InitWheel();
	// make wheel constraint pointing up and towards center
	plCar01_entity._wheelFL__016 = phcon_add(PH_WHEEL, plCar01_entity, my);
	phcon_setparams1(plCar01_entity._wheelFL__016, my.x, plCar01_vecUp, plCar01_vecRight);
	phcon_setparams2(plCar01_entity._wheelFL__016, vector(0,0,0), nullvector,
							vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
	my.passable=off;
}
action PlCar01_WheelFR
{
	my.passable=on;
	while (!plCar01_entity) { wait(1);}
	plCar01_FR=my;
	PlCar01_InitWheel();
	// make wheel constraint pointing up and towards center
	plCar01_entity._wheelFR__016 = phcon_add(PH_WHEEL, plCar01_entity, my);
	phcon_setparams1(plCar01_entity._wheelFR__016, my.x, plCar01_vecUp, plCar01_vecRight);
	phcon_setparams2(plCar01_entity._wheelFR__016, vector(0,0,0), nullvector,
							vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));

	my.passable=off;
}
action PlCar01_WheelRL
{
	my.passable=on;
	while (!plCar01_entity) { wait(1);}
	plCar01_RL=my;
	PlCar01_InitWheel();
	// make wheel constraint pointing up and towards center
	plCar01_entity._wheelRL__016 = phcon_add(PH_WHEEL, plCar01_entity, my);
	phcon_setparams1(plCar01_entity._wheelRL__016, my.x, plCar01_vecUp, plCar01_vecRight);
	phcon_setparams2(plCar01_entity._wheelRL__016, vector(0,0,0), nullvector,
							vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
	my.passable=off;
}
action PlCar01_WheelRR
{
	my.passable=on;
	while (!plCar01_entity) { wait(1);}
	plCar01_RR=my;
	PlCar01_InitWheel();
	// make wheel constraint pointing up
	plCar01_entity._wheelRR__016 = phcon_add(PH_WHEEL, plCar01_entity, my);
	phcon_setparams1(plCar01_entity._wheelRR__016, my.x, plCar01_vecUp, plCar01_vecRight);
	phcon_setparams2(plCar01_entity._wheelRR__016, vector(0,0,0), nullvector,
							vector(plCar01_entity._suspensionERP__016, plCar01_entity._suspensionCFM__016,0));
	my.passable=off;
}



//////////////////////////////////////////////////////////////
// construct car

// Desc: create the physics model of the car chassis
function plCar01_Phy_Init()
{
		my.passable=on;

		ph_fps_max_lock = fps_max;

		vec_set(plCar01_vecRight,vector(0,-1,0));	// reset right vector (in case of level restart)
		vec_rotate(plCar01_vecRight, my.pan); // rotate right vector to correspond to car orientation
		phent_settype(my, PH_RIGID, PH_BOX);
		phent_setmass(my, my._massChassis__016, PH_BOX);
		phent_setgroup(my, 2); // all car parts in group2 ->no Collisions
		phent_setfriction(my, my._fricChassis__016);
		phent_setdamping(my, my._dampChassis__016,my._dampChassis__016);
		phent_setelasticity(my, 10, 100); // little bouncing
	 	plCar01_entity=my;

		// set event handler
		plCar01_entity.event= PlCar01_Event;
		plCar01_entity.enable_friction=on;
		plCar01_entity.enable_scan=on;
		plCar01_entity.enable_shoot=on;
		plCar01_entity.enable_impact=on;


		vec_set(temp, my.x);
		while (plCar01_wheelCounter<4) { wait(1); } // wait till all parts are there
		my.passable=off;
		ph_setgravity(vector(0,0,-my._gravity__016));
}



// Desc: remove physical car model
function plCar01_Phy_Remove()
{
	plCar01_wheelCounter=0; //reset wheelcount if reloading
	plCar01_entity._targetSpeed__016=0; plCar01_entity._angSpeed__016=0;
	ph_setgravity(vector(0,0,0));
	phcon_remove(plCar01_entity._wheelFL__016); phcon_remove(plCar01_entity._wheelFR__016);
	phcon_remove(plCar01_entity._wheelRL__016); phcon_remove(plCar01_entity._wheelRR__016);
	phent_settype(plCar01_entity, 0,0);
	phent_settype(plCar01_FL, 0,0); phent_settype(plCar01_FR, 0,0);
	phent_settype(plCar01_RL, 0,0); phent_settype(plCar01_RR, 0,0);
}




// action: PlCar01
// title: Player Car Entity
// Desc: Action attached to the player entity
// Desc: Note: only valid if used once per level!
//
// uses _maxTorque__016, _maxTorqueBrake__016, _maxAngSpeed__016,
// uses _massWheel__016, _massChassis__016,
// uses _fricWheel__016, _fricChassis__016,
// uses _dampChassis__016, _dampWheel__016,
// uses _suspensionERP__016, _suspensionCFM__016,
// uses _gravity__016, _health__016
//
//------------------
// section: Heath
// cntl: spin 1 x 5
// help: Reduced by damage. When <=0, player dies.
//	skill1: Health 100
//
//------------------
// section: Physics Constants
// cntl: spin 1 x 5
// help: Maximum torque applied to rear wheels.
//	skill2: maxTorque 900
//
// cntl: spin 1 x 5
// help: Maximum torque applied to rear wheels while braking.
//	skill3: maxTorqueBrake 1000
//
// cntl: spin 1 x 5
// help: Maximum angular speed of wheels;
// help: max linear speed depends on wheel radius.
//	skill4: maxAngleSpeed 60
//
// cntl: spin 1 x 5
// help: The physical mass of each wheel.
//	skill5: massWheel 30
//
// cntl: spin 1 x 5
// help: The physical mass of the chassis.
// help: Note: mass be lighter than wheels so it doesn't tilt.
//	skill6: massChassis 10
//
// cntl: spin 1 x 5
// help: Friction of the wheels.
//	skill7: WheelFriction 100
//
// cntl: spin 1 x 5
// help: Friction of the chassis.
//	skill8: ChassisFriction 20
//
// cntl: spin 1 x 5
// help: Damping value for lin/ang chassis movement.
//	skill9: ChassisDamp 5
//
// cntl: spin 1 x 5
// help: Damping value for lin/ang wheel movement.
//	skill10: WheelDamp 15
//
// cntl: spin 0 100000 5
// help: Speed shocks are returned to correct position.
//	skill11: SuspensionERP 95000
//
// cntl: spin 0 100000 5
// help: How far shocks can overshoot.
//	skill12: SuspensionCFM 250
//
// cntl: spin 0 x 5
// help: Gravity's effect on this object.
//	skill13: Gravity 1800
//
action PlCar01
{
	// this is the action attached to the car
	plCar01_entity = my;

	if(plCar01_entity._health__016 <-0) { plCar01_entity._health__016 = 100; }

	// build a physics car and wait til all wheels are ready
	plCar01_Phy_Init();
	while (plCar01_wheelCounter<4) { wait(1); }

	PlCar01_Loop();	// run main update loop
}


endif;