ifndef props_wdl;
define props_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Props (00)
// class: OBJECT
// type: USER
// image: props00.pcx
// help: Handle "props" (think "movie props"). Props are items that can be placed in the level.
//
//
// needs: gid01.wdl, plSelect.wdl, trigger00.wdl
//
// external:
//
// prefix: props00_
// idcode: 021
//
// ENDHEADER
// ---------------------------------------------------------------------


// health
define	_health__021,SKILL1;      // damage this object can take before being destroyed
											  // if <= 0, object can not be destroyed

// control apperence
define	_alpha__021,SKILL2;

// rotate (degrees per tick)
define	_rotate_pan__021,SKILL4;
define	_rotate_tilt__021,SKILL5;
define	_rotate_roll__021,SKILL6;

// light effect
define	_lightrange__021,SKILL7;
define	_red__021,SKILL8;
define	_green__021,SKILL9;
define	_blue__021,SKILL10;
define	_light_pulse__021,SKILL11;	// rate of change of pulsing/activated light

define	_use_trigger__021,SKILL12;	// activate when trigger is set

// animation
define	_anim_base__021,SKILL13;	// base (non-animating) frame
define	_anim_start__021,SKILL14;	// starting frame while animating
define	_anim_end__021,SKILL15;		// ending frame while animating
define	_anim_rate__021,SKILL16;	// animation speed (negitive if only once)


// face player?
define	_facing__021,FLAG1;
// billboard?
define	_billboard__021,FLAG2;

define	_activate_rotation__021,FLAG3;   // activate rotation on trigger?
define	_activate_light__021,FLAG4; 		// activate light on trigger?
define	_activate_anim__021,FLAG5;			// activate animation on trigger?


// ---------------------------------------------------------------------

// Desc: handle basic events
function Props00_Event1()
{
	// shooting event
	if((EVENT_TYPE == event_shoot) && (gid01_event_type == gid01_event_projectile_damage))
	{
		my._health__021 -= gid01_event_value;
	}

	// activation scan
	// ++

}


// Desc: update the rotation of this entity
function	Props00_Update_Rotation()
{
	if((my._activate_rotation__021 == OFF) || (my._use_trigger__021 <= 0))
	{
		while(1) // while entity exisits
		{
			my.pan += TIME * my._rotate_pan__021;
			my.tilt += TIME * my._rotate_tilt__021;
			my.roll += TIME * my._rotate_roll__021;
			wait(1);
		}
	}
	else
	{
		// only rotate when activate
		while(1) // while entity exisits
		{
			if(Trigger00_Check(my._use_trigger__021,trigger00_bitlist) == 1)
			{
				my.pan += TIME * my._rotate_pan__021;
				my.tilt += TIME * my._rotate_tilt__021;
				my.roll += TIME * my._rotate_roll__021;
			}
  			wait(1);
		}

	}

}

// Desc: update the lighting of this entity
function Props00_Update_Lighting()
{
	my.red = my._red__021;
	my.green = my._green__021;
	my.blue = my._blue__021;

	if(my._activate_light__021 == OFF) // light not triggered
	{
		if(my._light_pulse__021 > 0)
		{
			// pulsing light effect
			while(1)
			{
				while(my.lightrange < my._lightrange__021)
				{
					wait(1);
					my.lightrange += my._light_pulse__021 * TIME;
				}
				my.lightrange = my._lightrange__021;
				while(my.lightrange > 0)
				{
					wait(1);
					my.lightrange -= my._light_pulse__021 * TIME;
				}
				my.lightrange = 0;
				wait(1);
			}
		}

		// my._light_pulse__021 <= 0  && my._activate_rotation__021 == OFF
		my.lightrange = my._lightrange__021;
		return; // nothing else to do
	}

	// triggered lights
	while(1)
	{
 		if(Trigger00_Check(my._use_trigger__021,trigger00_bitlist) == 1)
 		{
			my.lightrange = my._lightrange__021;
		}
		else
		{
			// light off
			my.lightrange = 0;
		}
		wait(1);
	}


}




// Desc: handle damage done to a prop
function Props00_Update_Damage()
{
	// set up event handler
	my.enable_shoot = on;  	// gun shoot
	my.enable_scan = on;		// explosions

	while(my._health__021 > 0)
	{
		wait(1);
	}


	// Handle Death
	if(my._health__021 <= 0)
	{
		// fade and remove
		my.transparent = on;
		while(my.alpha > 0)
		{
			my.alpha -= 3*time;
			wait(1);
		}

		proc_kill(1);	// stop the other functions spawned by this entity
		wait(1);       // ...let them finish their run
		ent_remove(me);
	}
}

// Desc: handle animation
function	Props00_Update_Animation()
{
	if(my._activate_anim__021 == OFF) // animation not triggered
	{
		my.frame = my._anim_start__021;
		while(1)
		{
			my.frame += abs(my._anim_rate__021) * TIME;
			if(my.frame > my._anim_end__021)
			{
				if(my._anim_rate__021 < 0) // once only?
				{
					my.frame = my._anim_end__021;
					return; // done
				}
				else
				{
					my.frame = my._anim_start__021 + (my.frame-my._anim_end__021);
				}
			}
			wait(1);
		}

	}
	else
	{
		while(1)
		{
 			while(Trigger00_Check(my._use_trigger__021,trigger00_bitlist) == 1)
 			{
				my.frame += abs(my._anim_rate__021) * TIME;
				if(my.frame > my._anim_end__021)
				{
					if(my._anim_rate__021 < 0) // once only?
					{
						my.frame = my._anim_end__021;
						return; // done
					}
					else
					{
						my.frame = my._anim_start__021 + (my.frame-my._anim_end__021);
					}
				}
				wait(1);
			}
			wait(1);
		}
	}
}

// action: Props00_Basic
// title: Basic Prop
// desc: Basic prop action.
// desc: Can be destroyed (if given health).
// desc: Can rotate, emit light, face the player, and/or be activated by triggers.
//
//
// uses _rotate_pan__021, _rotate_tilt__021, _rotate_roll__021,
// uses _alpha__021, _health__021
// uses _lightrange__021, _red__021, _green__021, _blue__021, _light_pulse__021
// uses _anim_base__021, _anim_start__021, _anim_end__021, _anim_rate__021
// uses _facing__021, _billboard__021, _use_trigger__021
// uses _activate_rotation__021, _activate_light__021, _activate_anim__021
//-----------------------------------------------------------------------------
//
// help: damage this object can take before being destroyed
// help: if <= 0, object can not be destroyed
// cntl: spin -1 x x
// skill1: Health -1
// skill2: Alpha 100
//
// section: Trigger
//
// help: Activate when trigger is set? (0 if always active)
// skill12: ID 0
//
// flag3: Rotate 0
// flag4: Light 0
// flag5: Animate 0
//
//
// section: Rotation (degrees per tick)
// skill4: pan 0
// skill5: tilt 0
// skill6: roll 0
//
// help: Turn to face the player at all time?
// flag1: Facing 0
//
// flag2: Billboard 0
//
// section: Lighting
// skill7: Range 0
// skill8: Red 0
// skill9: Green 0
// skill10: Blue 0
//
// help: Rate of change of pulsing/activated light
// skill11: Pulse 0
//
// section: Animation
// help: Base (non-animating) frame
// skill13: Base 0
//
// help: Starting frame while animating
// skill14: Start 0
//
// help: Ending frame while animating
// skill15: End 0
//
// help: Animation speed (negitive if only used once, otherwise it cycles)
// skill16: Rate 1
//
action Props00_Basic
{
	// facing?
	if(my._facing__021 == ON) { my.facing = on; }
	// billboard?
	if(my._billboard__021 == OFF) { my.oriented = ON; }

 	// set alpha transparency
	if(my._alpha__021 > 0)
	{
		my.transparent = on;
		my.alpha = my._alpha__021;
	}

	// set up event handler
	my.event = Props00_Event1;	// handle events

	// handle rotations
	if(  (my._rotate_pan__021 != 0)
	  || (my._rotate_tilt__021 != 0)
	  || (my._rotate_roll__021 != 0) )
	{
		Props00_Update_Rotation();
	}

	// handle lighting
	if(my._lightrange__021 > 0)
	{
		Props00_Update_Lighting();
	}

	// handle damage
	if(my._health__021 > 0)
	{
		Props00_Update_Damage();
	}

	// handle animation
	if(my._anim_base__021 >= 1)
	{
		my.frame = my._anim_base__021;
	}
	if(my._anim_rate__021 != 0)
	{
		Props00_Update_Animation();
	}

}



endif; // props00_wdl