ifndef saveLoad_wdl;
define saveLoad_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Save / Load Game State
// class: INTERFACE
// type: SUPPORT
// image: saveload.pcx
// help: Allows player to save current game state to one of 8 'slots'.
// help:Player can load a saved game state from a previously used slot.
//
//
// prefix: saveLoad00_
// idcode: 018
//
// needs: miscInput01.wdl
//
// ENDHEADER
// ---------------------------------------------------------------------





// entry: Save Game Directory
// id: 1
string	saveLoad00_dir_str = "";	// save game directory (pathed from game directory)

// entry: Save Game Basename
// entry_help: This name plus the slot number will be used to save states
// id: 2
string	saveLoad00_basename_str = "savegame";

// fixed number of save games
define	saveLoad00_slot_num = 8;

// entry: Cancel save/load key
// id: 4
string saveLoad00_esc_key_c = "esc";
var 	saveLoad00_esc_scancode = 0;

// entry: Save mode
// help: See C-Script documentation for details (0 is default)
// id: 5
var	saveLoad00_savemode = 0;

// entry: Save game mouse bitmap
// id: 6
bmap	saveLoad00_save_mouse_bmp = <arrow_red.pcx>;
// entry: Load game mouse bitmap
// id: 7
bmap	saveLoad00_load_mouse_bmp = <arrow_blue.pcx>;
bmap*	saveLoad00_tmp_mouse_bmp;   			// used to temp store old mousemap



//----------------------------------------------------------------------
// section: Save Game Panel

// entry: The "Save Game" Background panel bitmap
// id: 8
bmap saveLoad00_save_bmp = <saveLoad00_save_bmp.pcx>;

// enable: Auto Center Panel?
// id: 9
define saveLoad00_auto_center_save;


panel saveLoad00_save_pan
{
// entry: Layer
// id: 10
	layer = 10;
// entry: Pos X
// help: Position X (only valid if auto center is off)
// id: 11
	pos_x = 0;
// entry: Pos Y
// help: Position Y (only valid if auto center is off)
// id: 12
	pos_y = 0;

	bmap = saveLoad00_save_bmp;

 	flags = refresh;
}

//----------------------------------------------------------------------
// section: Load Game Panel

// entry: The "Load Game" Background panel bitmap
// id: 13
bmap saveLoad00_load_bmp = <saveLoad00_load_bmp.pcx>;

// enable: Auto Center Panel?
// id: 14
define saveLoad00_auto_center_load;


panel saveLoad00_load_pan
{
// entry: Layer
// id: 15
	layer = 10;
// entry: Pos X
// help: Position X (only valid if auto center is off)
// id: 16
	pos_x = 0;
// entry: Pos Y
// help: Position Y (only valid if auto center is off)
// id: 17
	pos_y = 0;

	bmap = saveLoad00_load_bmp;

 	flags = refresh;
}


//----------------------------------------------------------------------
// section: save/load slot buttons

// entry: Empty slot bitmap
// entry_help: Bitmap used to represent an empty slot
// id: 18
bmap saveLoad00_empty_bmp = <saveLoad00_empty_bmp.pcx>;

// entry: Full slot bitmap
// entry_help: Bitmap used to represent an full slot
// id: 19
bmap saveLoad00_full_bmp = <saveLoad00_full_bmp.pcx>;

// entry: Start X
// help: Starting position of first save/load slot (offset from panel pos)
// id: 20
define	saveLoad00_slot_pos_x = 10;
// entry: Start Y
// help: Starting position of first save/load slot (offset from panel pos)
// id: 21
define	saveLoad00_slot_pos_y = 52;


// save/load slot function prototypes
function saveLoad00_slot1_click();
function saveLoad00_slot2_click();
function saveLoad00_slot3_click();
function saveLoad00_slot4_click();
function saveLoad00_slot5_click();
function saveLoad00_slot6_click();
function saveLoad00_slot7_click();
function saveLoad00_slot8_click();

// variable used to indicate that a slot was clicked.
// contains the number of the slot clicked or 0 if waiting
var	saveLoad00_slot_clicked;


panel saveLoad00_slot1_pan
{
// entry: Layer
// id: 22
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot1_click;
}
panel saveLoad00_slot2_pan
{
// entry: Layer
// id: 23
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot2_click;
}
panel saveLoad00_slot3_pan
{
// entry: Layer
// id: 24
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot3_click;
}
panel saveLoad00_slot4_pan
{
// entry: Layer
// id: 25
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot4_click;
}
panel saveLoad00_slot5_pan
{
// entry: Layer
// id: 26
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot5_click;
}
panel saveLoad00_slot6_pan
{
// entry: Layer
// id: 27
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot6_click;
}
panel saveLoad00_slot7_pan
{
// entry: Layer
// id: 28
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot7_click;
}
panel saveLoad00_slot8_pan
{
// entry: Layer
// id: 29
	layer = 11;
	bmap = saveLoad00_empty_bmp;
 	flags = overlay,refresh;
	on_click = saveLoad00_slot8_click;
}



// Used to tell the calling function when we are finished displaying save/load menus
// 1 - active, 0 - done
var saveLoad00_active = 0;

// array of slot states, 1-slot is used, 0-slot is empty
var	saveLoad00_slot_state[saveLoad00_slot_num];

string saveLoad00_tmp_str[256]; // temp value string
string saveLoad00_tmp2_str[256]; // temp value string


// ---------------------------------------------------------------------
// Basic Functions


// Desc: init all values
starter SaveLoad00_Init()
{
	diag("\nWDL-Loaded:saveLoad00.wdl");


	/////////////////////////////
	// Set up keyboard scancodes
	/////////////////////////////
	// use MiscInput01_Key_For_Str() to convert strings into scancodes
	// next item down key
	str_cpy(miscInput01_temp_str,saveLoad00_esc_key_c);
	saveLoad00_esc_scancode = MiscInput01_Key_For_Str();


	if(saveLoad00_savemode == 0)
	{
		saveLoad00_savemode = (SV_ALL-SV_INFO);
	}
}


// Desc: save the current state using the file name given
// Input: name (string) name of the save game (including path)
//			 num (var) number appendend to save game name
//			 mode the save mode (see game_save documentation)
// Return: -1 on error, 1 on success
function SaveLoad00_SaveState(name, num, mode)
{
	if(game_save(name,num,mode) <= 0) { return(-1); }       // error occured durring save
	else { return(1); }							// state save successful
}


// Desc: Load a game state from a file name given.
// Input: name (string)
// Return: -1 on error
function SaveLoad00_LoadState(name, num)
{
	if(game_load(name,num) <= 0) {  return(-1); }       // error occured durring load
	// success
}

// Desc: Set the save game directory
//+++


// ---------------------------------------------------------------------
// "Slot" functions

// Desc: functions called when the player clicks on slots
function saveLoad00_slot1_click() { saveLoad00_slot_clicked = 1; }
function saveLoad00_slot2_click() { saveLoad00_slot_clicked = 2; }
function saveLoad00_slot3_click() { saveLoad00_slot_clicked = 3; }
function saveLoad00_slot4_click() { saveLoad00_slot_clicked = 4; }
function saveLoad00_slot5_click() { saveLoad00_slot_clicked = 5; }
function saveLoad00_slot6_click() { saveLoad00_slot_clicked = 6; }
function saveLoad00_slot7_click() { saveLoad00_slot_clicked = 7; }
function saveLoad00_slot8_click() { saveLoad00_slot_clicked = 8; }


function	SaveLoad00_Slot_Check()
{
	// check for current save games
	var	count;
	var	handle;
	count = 0;
	while(count < saveLoad00_slot_num)
	{
		// create file name: basename + count
		str_for_num(saveLoad00_tmp2_str,(count+1));
		str_cpy(saveLoad00_tmp_str,saveLoad00_basename_str);
		str_cat(saveLoad00_tmp_str,saveLoad00_tmp2_str);
		str_cat(saveLoad00_tmp_str,".SAV");

		handle = File_Open_Read(saveLoad00_tmp_str);
		if(handle == 0)
		{
			// not a valid save game
			saveLoad00_slot_state[count] = 0;
		}
		else
		{
			// savegame exisits
			saveLoad00_slot_state[count] = 1;
			File_Close(handle);
		}

		count+=1;
	}

}

// Desc: set all 8 of the slot bitmaps to the correct type (empty or full)
//  based on the information stored in saveLoad00_slot_state.
//			Also set the position relitive to the values passed in
function	SaveLoad00_Slot_Set_All(base_x, base_y)
{
	if(saveLoad00_slot_state[0] == 1)
	{
		saveLoad00_slot1_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot1_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot1_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot1_pan.pos_y = base_y + saveLoad00_slot_pos_y;
//--
	if(saveLoad00_slot_state[1] == 1)
	{
		saveLoad00_slot2_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot2_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot2_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot2_pan.pos_y = base_y + saveLoad00_slot_pos_y + bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[2] == 1)
	{
		saveLoad00_slot3_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot3_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot3_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot3_pan.pos_y = base_y + saveLoad00_slot_pos_y + 2*bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[3] == 1)
	{
		saveLoad00_slot4_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot4_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot4_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot4_pan.pos_y = base_y + saveLoad00_slot_pos_y + 3*bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[4] == 1)
	{
		saveLoad00_slot5_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot5_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot5_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot5_pan.pos_y = base_y + saveLoad00_slot_pos_y + 4*bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[5] == 1)
	{
		saveLoad00_slot6_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot6_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot6_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot6_pan.pos_y = base_y + saveLoad00_slot_pos_y + 5*bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[6] == 1)
	{
		saveLoad00_slot7_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot7_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot7_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot7_pan.pos_y = base_y + saveLoad00_slot_pos_y + 6*bmap_height(saveLoad00_empty_bmp);
//--
	if(saveLoad00_slot_state[7] == 1)
	{
		saveLoad00_slot8_pan.bmap = saveLoad00_full_bmp;
	}
	else
	{
		saveLoad00_slot8_pan.bmap = saveLoad00_empty_bmp;
	}
	saveLoad00_slot8_pan.pos_x = base_x + saveLoad00_slot_pos_x;
	saveLoad00_slot8_pan.pos_y = base_y + saveLoad00_slot_pos_y + 7*bmap_height(saveLoad00_empty_bmp);
}

// Desc: Show (1) or Hide (0) all slot panels
function	SaveLoad00_Slot_ShowHide_All(showhide)
{
	if(showhide == 1)
	{
		// show all slots
		saveLoad00_slot1_pan.visible = on;
		saveLoad00_slot2_pan.visible = on;
		saveLoad00_slot3_pan.visible = on;
		saveLoad00_slot4_pan.visible = on;
		saveLoad00_slot5_pan.visible = on;
		saveLoad00_slot6_pan.visible = on;
		saveLoad00_slot7_pan.visible = on;
		saveLoad00_slot8_pan.visible = on;

	}
	else
	{
		// hide all slots
		saveLoad00_slot1_pan.visible = off;
		saveLoad00_slot2_pan.visible = off;
		saveLoad00_slot3_pan.visible = off;
		saveLoad00_slot4_pan.visible = off;
		saveLoad00_slot5_pan.visible = off;
		saveLoad00_slot6_pan.visible = off;
		saveLoad00_slot7_pan.visible = off;
		saveLoad00_slot8_pan.visible = off;
 	}


}


// Desc: Display save game panel, allow user the option to save game state
function SaveLoad00_SaveState_Display()
{
	// Display panel
	ifdef saveLoad00_auto_center_save;
	//if(saveLoad00_auto_center_save == 1)
	//{
		// center the panel
 		saveLoad00_save_pan.pos_x = (screen_size.x - bmap_width(saveLoad00_save_bmp)) / 2;
  		saveLoad00_save_pan.pos_y = (screen_size.y - bmap_height(saveLoad00_save_bmp)) / 2;
	//}
	endif;
	saveLoad00_save_pan.visible = on;

	// set up 'slots'
	SaveLoad00_Slot_Check();	// check if slots are used
	SaveLoad00_Slot_Set_All(saveLoad00_save_pan.pos_x, saveLoad00_save_pan.pos_y);    		// set the state of all slots
	SaveLoad00_Slot_ShowHide_All(1);  	// show all slots

	// save mouse state and bring up mouse cursor
	var	old_mouse_mode;
	old_mouse_mode = mouse_mode;
	saveLoad00_tmp_mouse_bmp = mouse_map;
	mouse_mode = 2;
	mouse_map = saveLoad00_save_mouse_bmp;


	saveLoad00_slot_clicked = 0;	// wait for user to click slot
	// loop on user input
	while(saveLoad00_slot_clicked == 0)
	{
		if(Key_Pressed(saveLoad00_esc_scancode)) { break; }	// cancel save
		mouse_pos.x = pointer.x;
		mouse_pos.y = pointer.y;
		wait(1);
	}

	// restore mouse mode and bitmap
	mouse_mode = old_mouse_mode;
	mouse_map = saveLoad00_tmp_mouse_bmp;


	SaveLoad00_Slot_ShowHide_All(0);  	// hide all slots

	saveLoad00_save_pan.visible = off;
	saveLoad00_active = 0; 	// signal calling function that we are done


	// save game
	if((saveLoad00_slot_clicked > 0) && (saveLoad00_slot_clicked < 9))
	{
		wait(1);	// allow menus to vanish...
		SaveLoad00_SaveState(saveLoad00_basename_str,saveLoad00_slot_clicked,saveLoad00_savemode);
	}
}


// Desc: Display load game panel, allow user the option to load a game state
function SaveLoad00_LoadState_Display()
{
	// Display panel
	ifdef saveLoad00_auto_center_load;
	//if(saveLoad00_auto_center_load == 1)
	//{
		// center the panel
 		saveLoad00_load_pan.pos_x = (screen_size.x - bmap_width(saveLoad00_load_bmp)) / 2;
  		saveLoad00_load_pan.pos_y = (screen_size.y - bmap_height(saveLoad00_load_bmp)) / 2;
	//}
	endif;
	saveLoad00_load_pan.visible = on;

	// set up 'slots'
	SaveLoad00_Slot_Check();	// check if slots are used
	SaveLoad00_Slot_Set_All(saveLoad00_load_pan.pos_x, saveLoad00_load_pan.pos_y);    		// set the state of all slots
	SaveLoad00_Slot_ShowHide_All(1);  	// show all slots

	// save mouse state and bring up mouse cursor
	var	old_mouse_mode;
	old_mouse_mode = mouse_mode;
	saveLoad00_tmp_mouse_bmp = mouse_map;
	mouse_mode = 2;
	mouse_map = saveLoad00_load_mouse_bmp;


	saveLoad00_slot_clicked = 0;	// wait for user to click slot
	// loop on user input
	while(saveLoad00_slot_clicked == 0)
	{
		if(Key_Pressed(saveLoad00_esc_scancode)) { break; }	// cancel save
		mouse_pos.x = pointer.x;
		mouse_pos.y = pointer.y;
		wait(1);
	}

	// restore mouse mode and bitmap
	mouse_mode = old_mouse_mode;
	mouse_map = saveLoad00_tmp_mouse_bmp;


	SaveLoad00_Slot_ShowHide_All(0);  	// hide all slots

	saveLoad00_load_pan.visible = off;
	saveLoad00_active = 0; 	// signal calling function that we are done

	// load game
	if((saveLoad00_slot_clicked > 0) && (saveLoad00_slot_clicked < 9))
	{
		wait(1);
		SaveLoad00_LoadState(saveLoad00_basename_str,saveLoad00_slot_clicked);
	}

}

endif;