ifndef sky_wdl;
define sky_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Day/Night Animated Sky (00)
// class: ENVIR
// type: USER
// image: sky00.pcx
// help: Create 3 sky spheres (with animation) and 2 scene maps.
//
//
// prefix: sky00_
// idcode: 01A
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// section: Sky Sphere 0 (night sky)
sky sky00_00_sky
{
// entry: Sky image
// id: 1
	type = <sky_stars.pcx>;
// entry: Speed X
// help: Animation speed in x
// id: 2
	speed_u = 1.5;
// entry: Speed Y
// help: animation speed in y
// id: 3
	speed_v = 0.0;

// entry: Scale X
// help: Scale in X dimention
// id: 4
	scale_x = 0.125;
// entry: Scale Y
// help: Scale in y dimention
// id: 5
	scale_y = 0.125;

// entry_drop: Sky type
// help: Select render style of the sky (dome, cube, cylinder)
// cntl: drop_list dome cube cylinder
// id: 30
	flags = dome;

// entry: Layer
// id: 6
	layer = 1;
}

// section: Sky Sphere 1 (day sky)
sky sky00_01_sky
{
// entry: Sky image
// id: 7
	type = <sky_blue.pcx>;
// entry: Speed X
// help: Animation speed in x
// id: 8
	speed_u = 1.5;
// entry: Speed Y
// help: Animation speed in y
// id: 9
	speed_v = 0.5;

// entry: Scale X
// help: Scale in X dimention
// id: 10
	scale_x = 1.0;
// entry: Scale Y
// help: Scale in Y dimention
// id: 11
	scale_y = 1.0;

// entry_drop: Sky type
// help: Select render style of the sky (dome, cube, cylinder)
// cntl: drop_list dome cube cylinder
// id: 31
	flags = dome, visible, transparent;

	alpha = 100;

// entry: Layer
// id: 12
	layer = 2;
}

// section: Sky Sphere 2 (cloud layer)
sky sky00_02_sky
{
// entry: sky image
// id: 13
	type = <sky_cloud.pcx>;
// entry: Speed X
// help: Animation speed in x
// id: 14
	speed_u = 3.5;
// entry: Speed Y
// help: Animation speed in y
// id: 15
	speed_v = 4.0;

// entry: Scale X
// help: Scale in X dimention
// id: 16
	scale_x = 1.0;
// entry: Scale Y
// help: Scale in Y dimention
// id: 17
	scale_y = 1.0;

// entry_drop: Sky type
// help: Select render style of the sky (dome, cube, cylinder)
// cntl: drop_list dome cube cylinder
// id: 32
	flags = dome,overlay,transparent, visible;  // sky dome, black transparent, alpha

	alpha = 75;

// entry: Layer
// id: 18
	layer = 3;
}

// Section: Nighttime horizon
sky sky00_00_scene
{
// entry: Scenemap image
// id: 19
	type = <night_mount.pcx>;

// entry: Scale X
// help: How much horizon is taken by one image (1 = full 360 degrees)
// id: 20
	scale_x = 0.125;

// entry: Pan
// help: Rotate along the horizon (pan)
// id: 21
	pan = 0;
// entry: Tilt
// help: Rotate vertically (tilt)
// id: 22
	tilt = -3; // moves it

	flags = scene;	// backdrop scene
// entry: Layer
// id: 23
	layer = 4; // display above sky entities with lower layers
}

// section: Daytime horizon
sky sky00_01_scene
{
// entry: scenemap image
// id: 24
	type = <sky_mount.pcx>;

// entry: Scale X
// help: How much horizon is taken by one image (1 = full 360 degrees)
// id: 25
	scale_x = 0.125;

// entry: Pan
// help: Rotate along the horizon (pan)
// id: 26
	pan = 0;
// entry: Tilt
// help: Rotate vertically (tilt)
// id: 27
	tilt = -3; // moves it

	flags = scene,transparent,visible; // backdrop scene, alpha, visible

// entry: Layer
// id: 28
	layer = 5; // display above sky entities with lower layers

	alpha = 100;
}


// ---------------------------------------------------------------------
// Local Vars
var sky00_day_or_night = 1;	// 1 for day, 0 for night, otherwise trans

// ---------------------------------------------------------------------
// Functions

starter Sky00_Load()
{
	diag("\nWDL-Loaded:Sky00.wdl");
}


// Desc: transition from daytime (sky01) to nighttime (sky00) in the time
//  given by duration.
function Sky00_Day_To_Night(duration)
{
	var	trans;

	// not day or transitioning
	if(sky00_day_or_night != 1) { return(-1); }

	// turn on night layers
	sky00_00_sky.visible = on;    // nighttime sky
	sky00_00_scene.visible = on;	// nighttime horizon

	if(duration > 0) // make sure duration is valid
	{
		sky00_day_or_night = 0.5;	// flag that we are transitioning

		trans = duration;

		// fade to night
		while(trans > 0)
		{
			sky00_01_sky.alpha = 100*(trans/duration);
			sky00_01_scene.alpha = sky00_01_sky.alpha;
			trans -= time;
			wait(1);
		}

  		sky00_day_or_night = 0; // flag that we are in night mode
	}

	// turn off day layers
	sky00_01_sky.alpha = 0;
	sky00_01_scene.alpha = 0;
	sky00_01_sky.visible = off;
	sky00_01_scene.visible = off;
}

// Desc: transition from nighttime (sky00) to daytime (sky01) in the time
//  given by duration.
function Sky00_Night_To_Day(duration)
{
	var	trans;

	// not night or transitioning
	if(sky00_day_or_night != 0) { return(-1); }

	// turn on day layers
	sky00_01_sky.visible = on;    // nighttime sky
	sky00_01_scene.visible = on;	// nighttime horizon

	// catch invalid values
	if(duration > 0)
	{
		sky00_day_or_night = 0.5;	// flag that we are transitioning

		trans = 0;

		// fade to day
		while(trans < duration)
		{
			sky00_01_sky.alpha = 100*(trans/duration);
			sky00_01_scene.alpha = sky00_01_sky.alpha;
			trans += time;
			wait(1);
		}

  		sky00_day_or_night = 1; // flag that we are in day mode
	}

	sky00_01_sky.alpha = 100;
	sky00_01_scene.alpha = 100;
	// turn off night
	sky00_00_sky.visible = off;
	sky00_00_scene.visible = off;
}


// Desc: Fade the cloud layer from its current alpha to a target alpha in
//  the time given by duration.
function	Sky00_Fade_Clouds(target,duration)
{
	var	trans;
	var	delta;
	var	start;

	// catch invalid values
	if(target < 0) 	{ target = 0; }  	// min alpha
	if(target > 100) 	{ target = 100; }	// max alpha
	if(duration <= 0)                   // duration not valid
	{
		sky00_02_sky.alpha = target;
		return;
	}


	trans = 0;
	start = sky00_02_sky.alpha;
	delta = target - sky00_02_sky.alpha;  // change in alpha

	while(trans < duration)
	{
		sky00_02_sky.alpha = start + delta*(trans/duration);
		trans += time;
		wait(1);
	}
	sky00_02_sky.alpha = target;
}

endif;