ifndef teleport_wdl;
define teleport_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4
// date: 050412
//
// title: Teleporters (00)
// class: OBJECT
// type: USER
// image: teleport00.pcx
// help: Allows you to set up teleporters to teleport the player around a level.
// help: Note: If you need to teleport the player to different levels, use the "levels00.wdl" script.
//
//
// needs: gid01.wdl, plSelect.wdl, trigger00.wdl
//
// external:
//
// prefix: teleport00_
// idcode: 025
//
// ENDHEADER
// ---------------------------------------------------------------------


// entry: Max
// help: Max number of array targets in the level
// id: 1
define	teleport00_max_targets = 24;
// handles to teleporter targets
var		teleport00_target_handle[teleport00_max_targets];


// ---------------------------------------------------------------------
// Skills
define	_id__025,SKILL1;	// Teleporter/Target id (1-teleport00_max_targets)
define	_use_trigger__025,SKILL2;	// 0- always active, >0- active only when this trigger is set





// Desc: make sure the ID falls within the right range
// 	MY is "Teleport00_Target" or "Teleport00" entity
function	Teleport00_Cap_ID()
{
	if(my._id__025 <= 1)
	{
		my._id__025 = 0;
	}
	else
	{
		if(my._id__025 > teleport00_max_targets)
		{
			my._id__025 = teleport00_max_targets-1;
		}
		else  // in range
		{
			my._id__025 = int(my._id__025) - 1;
		}
	}
}

// ---------------------------------------------------------------------
// Actions


// action: Teleport00_Target
// title: Teleporter Target
// Desc: Target of a teleport00 (player will start here)
// uses: _id__025
//
// help: Teleporter/Target id
// cntl: spin 1 x 1
//	skill1: ID 1
action Teleport00_Target
{
	my.passable = on; my.invisible = on;

 	Teleport00_Cap_ID();	// cap the id skill into the correct range

	if(teleport00_target_handle[my._id__025] != 0)
	{
		diag("\nWARNING! Teleport00::Teleport00_Target- More then one Target with the same ID");
	}
	// set the array to this entity
	teleport00_target_handle[my._id__025] = handle(my);
}

// Find the target teleport pad.
function Teleport00_Event()
{
	// make sure this is the player
	if(YOU != plSelect_curr_ent) { return(0); }

	if(my._use_trigger__025 > 0)
	{
		if(Trigger00_Check(my._use_trigger__025,trigger00_bitlist) != 1)
		{
			// teleporter's trigger not set
			return(0);
		}

	}


	if(teleport00_target_handle[my._id__025] != 0)
	{
		my = ptr_for_handle(teleport00_target_handle[my._id__025]);
		wait(1);   // need wait for my assignment to work

 		// move player to entry point
 		vec_set(YOU.x,my.x);
		vec_set(YOU.pan,my.pan);
  	}
	else
	{
		diag("\nWARNING! Teleport00::Teleport00_Event- Bad teleport00_target_handle!");
		return(-1);
	}

	return(1);
}


// action: Teleport00
// title: Teleporter
// Desc: Teleports the player to a Teleport00_Target entity in this level with the same ID.
// uses: _id__025, _use_trigger__025
//
// help: Teleporter/Target id
// cntl: spin 1 x 1
//	skill1: ID 1
// help: 0- always active, >0- active only when the trigger with matching ID is set
// cntl: spin 0 x 1
// skill2: UseTrigger 0
//
action Teleport00
{
//	my.passable = off; my.invisible = off;
//	my.transparent = on; my.alpha = 55;    // ??? test only

 	Teleport00_Cap_ID();	// cap the id skill into the correct range

	my.event = Teleport00_Event;
	if(my.passable == ON) { my.enable_trigger = ON; }
	else { my.enable_impact = ON; }
}

endif;  //teleport00_wdl