ifndef weaponfx_wdl;
define weaponfx_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 4.1
// date: 050721
//
// title: Weaponfx (00)
// class: WEAPON
// type: SUPPORT
// image: weaponfx00.pcx
// help: Handle weapon fx (bullet holes, sparks, etc.).
//
//
// needs: gid01.wdl, trigger00.wdl, particle00.wdl
//
// external:
//
// prefix: weaponfx00_
// idcode: 020
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// section: Bullet Holes

// entry: Bullet hole bitmap filename
// id: 1
string weaponfx00_hole_str = <bulhole.tga>;

// entry: Durration (Seconds)
// help: Bullet hole durration (seconds)
// id: 2
define weaponfx00_hole_time = 10;

// entry: Max
// help: Max number of bullet holes visible on any level
// id: 3
define weaponfx00_max_holes = 100; 	// max number of entities
var weaponfx00_hole_array[weaponfx00_max_holes]; // array of hole entity handles
var weaponfx00_hole_index = 0;   // current index into hole array
entity* weaponfx00_hole_ent;		// temp pointer to hole entity

// Desc: function attached to created bullet holes
// Use: ent_create(weaponfx00_hole_str,TARGET,WeaponFX00_Hole)
function WeaponFX00_Hole()
{
	my.transparent = on;
 //	my.flare = on;
	my.passable = on;
	my.oriented = on;
	vec_to_angle(my.pan,NORMAL.x);	// orientate to wall normal
	vec_add(my.x,NORMAL.x);	// offset along normal


	// wrap arround array
	if(weaponfx00_hole_index >= weaponfx00_max_holes)
	{
		weaponfx00_hole_index = 0;
	}

	// clear current hole if needed
	if( weaponfx00_hole_array[weaponfx00_hole_index] != 0 )
	{
		// remove old
		weaponfx00_hole_ent = ptr_for_handle(weaponfx00_hole_array[weaponfx00_hole_index]);
		ent_remove(weaponfx00_hole_ent);
	}
	// store this new hole in the handle array
	weaponfx00_hole_array[weaponfx00_hole_index] = handle(me);

	// store index in entity (for auto removal)
	my.skill4 = weaponfx00_hole_index;

	weaponfx00_hole_index +=1;

	sleep(weaponfx00_hole_time);  // wait for set ammount of time

	// bullet hole "timed out", slowly fade away...
	while(my.alpha > 0)
	{
		my.alpha -= TIME;
		wait(1);
	}
	weaponfx00_hole_array[my.skill4] = 0;	// remove from array
	ent_remove(me);
}

// Desc: Remove all bullet holes
function WeaponFX00_Hole_Remove_All()
{
	weaponfx00_hole_index = 0;
	while(weaponfx00_hole_index < weaponfx00_max_holes)
	{
		if( weaponfx00_hole_array[weaponfx00_hole_index] != 0 )
		{
			// remove old
			weaponfx00_hole_ent = ptr_for_handle(weaponfx00_hole_array[weaponfx00_hole_index]);
			ent_remove(weaponfx00_hole_ent);
			weaponfx00_hole_array[weaponfx00_hole_index] = 0;
		}
		weaponfx00_hole_index += 1;
	}
}



//----------------------------------------------------------------------
// section: Spark effect.

// Desc: spark particle update
function WeaponFX00_Spark_Update()
{
	my.alpha -= 6*time;
	if (my.alpha <= 0) { my.lifespan = 0; }
}


// Desc: Draw sparks.
function WeaponFX00_Spark_FX()
{
// section: Draw spark effect.
// entry: Spark red value
// id: 10
	my.red = 255;
// entry: Spark green value
// id: 11
	my.green = 245;
// entry: Spark blue value
// id: 12
	my.blue = 200;
// entry: Spark size
// id: 13
	my.size = 2;

	Particle00_Randomize(my.vel_x,20);	// random vector
	my.alpha = 25 + random(25);
//	my.bmap = particle00_spark_bmp;
	my.flare = on;
	my.bright = on;
	my.beam = on;
	my.move = on;
	my.function = WeaponFX00_Spark_Update;
}


//----------------------------------------------------------------------
// section: Tracer effect.

// Desc: Draw 'tracer' rounds effect.
function WeaponFX00_Tracer_FX()
{
// entry: Tracer red value
// id: 14
	my.red = 255;
// entry: Tracer green value
// id: 15
	my.green = 250;
// entry: Tracer blue value
// id: 16
	my.blue = 155;
// entry: Tracer size
// id: 17
	my.size = 1;
// entry: Tracer duration (ticks)
// id: 18
	my.lifespan = 4;

	// get the length
	vec_diff(my.vel_x,my.vel_x,my.x);

	// 'chop' vector so it takes full lifespan to reach end.
	vec_scale(my.vel_x,(1/my.lifespan));

	my.alpha = 55 + random(15);
	my.flare = on;
	my.bright = on;
	my.beam = on;
	my.move = on;
  	my.function = WeaponFX00_Spark_Update;
}




//----------------------------------------------------------------------
// Draw "Laser"::


// Desc: draw line from start to end
// Note! Only works in Commercial edition or higher (streak effect).
function WeaponFX00_DrawLaser(&start,&end)
{
	vec_diff(temp,end,start);
	effect(Particle00_DrawLine_Func,1,start,temp);
	// add light

}





//----------------------------------------------------------------------
// Flash::

// Desc: create a light 'flash' at hitpoint
//
// use: ent_create(temp_ent_str,TARGET,WeaponFX00_Flash);
function WeaponFX00_Flash()
{
// section: Hit "flash" effect.
// entry: Hit "flash" red value
// id: 20
	my.red = 255;
// entry: Hit "flash" green value
// id: 21
	my.green = 200;
// entry: Hit "flash" blue value
// id: 22
	my.blue = 200;
// entry: Hit "flash" light range
// id: 23
	my.lightrange = 75;
// entry: Hit "flash" light durration (seconds)
// id: 24
	my.skill1 = 0.25;

	my.invisible = on; my.passable = on;


	sleep(my.skill1);
	wait(1); // at least one frame must pass
	ent_remove(me);
}


//----------------------------------------------------------------------
// section: Gun Brass

// entry: gun brass bitmap
// id: 25
bmap	weaponFX00_gbrass_bmp = <gbrass.tga>;

function WeaponFX00_Gunbrass_Update()
{
	if(my.vel_z > -16)
	{
		my.vel_z -= 2 * TIME;
	}
}

function WeaponFX00_Gunbrass_FX()
{
	my.vel_x = random(6)-6;	// -3 to 3
	my.vel_y = random(6)-6;  // -3 to 3
	my.vel_z = random(3)+8;  // 8 to 11

	my.bmap = weaponFX00_gbrass_bmp;

	my.move = on;
	my.function = WeaponFX00_Gunbrass_Update;
	my.lifespan = 36;
}


//----------------------------------------------------------------------
// section: Muzzle Flash

// entry: First muzzle flash bitmap (direction)
// id: 26
string	weaponFX00_mflash_str = <muzzleflash.tga>;

// entry: Second muzzle flash bitmap (facing)
// id: 50
string	weaponFX00_mflash2_str = <muzzleflash2.tga>;

// Desc: Create a third image that faces the camera (hiding the fact it is 2D)
function	WeaponFX00_MuzzleFlash_Facing()
{
	my.transparent = on;
	my.bright = on;
	my.unlit = on;
	my.passable = on;
	my.oriented = off;
	my.facing = on;

	my.skill1 = you.skill1;


	sleep(my.skill1);
	wait(1);	// at least one frame..
	ent_remove(me);
}

// Desc: create a second muzzle flash to cross with the first one
function WeaponFX00_MuzzleFlash_Cross()
{
	my.transparent = on;
	my.bright = on;
	my.unlit = on;
	my.passable = on;
	my.oriented = on;

	vec_set(my.pan,you.pan);
	my.tilt += 90;
	my.skill1 = you.skill1;


	sleep(my.skill1);
	wait(1);	// at least one frame..
	ent_remove(me);
}

// Desc: create a muzzle flash orientated towards the target (to)
function WeaponFX00_MuzzleFlash(&from,&to)
{
	me = ent_create(weaponFX00_mflash_str,from,null);

	my.transparent = on;
	my.bright = on;
	my.unlit = on;
	my.passable = on;
	my.oriented = on;

	vec_diff(temp,to,from);
	vec_to_angle(my.pan,temp.x);	// orientate towards target
	my.pan += 90;

// entry: Half Width
// help: Size of half the width of the flash image (used for offsets)
// id: 27
	my.skill2 = 32;

	// move forward by half image width
	vec_normalize(temp.x,my.skill2);
	vec_add(my.x,temp.x);


	// section: Muzzle flash effect.
// entry: Muzzle flash red value
// id: 28
	my.red = 255;
// entry: Muzzle flash green value
// id: 29
	my.green = 200;
// entry: Muzzle flash blue value
// id: 30
	my.blue = 200;
// entry:Muzzle flash light range
// id: 31
	my.lightrange = 55;
// entry:Muzzle flash durration (ticks)
// id: 32
	my.skill1 = 0.25;

	// create 2nd cross flash image
 	ent_create(weaponFX00_mflash_str,my.x,WeaponFX00_MuzzleFlash_Cross);

	// create facing flash image
 	ent_create(weaponFX00_mflash2_str,my.x,WeaponFX00_MuzzleFlash_Facing);

	sleep(my.skill1);
	wait(1);	// at least one frame...
	ent_remove(me);
}


// ---------------------------------------------------------------------
// Section: Sound

// Entry: Gun "empty" sound
// id: 33
sound		weaponfx00_empty_snd = <empty.wav>;
// Entry: Empty volume
// id: 34
define	weaponfx00_empty_vol = 55;

// Entry: Gun "reload" sound
// id: 35
sound		weaponfx00_reload_snd = <reload.wav>;
// Entry: Reload volume
// id: 36
define	weaponfx00_reload_vol = 55;

// Entry: Gunshot #1 sound
// id: 37
sound		weaponfx00_shot1_snd = <shot1.wav>;
// Entry: Gunshot #1 volume
// id: 38
define	weaponfx00_shot1_vol = 1275;
// Entry: Gunshot #2 sound
// id: 39
sound		weaponfx00_shot2_snd = <shot4.wav>;
// Entry: Gunshot #2 volume
// id: 40
define	weaponfx00_shot2_vol = 1275;
// Entry: Gunshot #3 sound
// id: 41
sound		weaponfx00_shot3_snd = <shot8.wav>;
// Entry: Gunshot #3 volume
// id: 42
define	weaponfx00_shot3_vol = 1275;
// Entry: Gunshot #4 sound
// id: 43
sound		weaponfx00_shot4_snd = <shot5.wav>;
// Entry: Gunshot #4 volume
// id: 44
define	weaponfx00_shot4_vol = 1275;
// Entry: Gunshot #5 sound
// id: 45
sound		weaponfx00_shot5_snd = <shot6.wav>;
// Entry: Gunshot #5 volume
// id: 46
define	weaponfx00_shot5_vol = 1275;
// Entry: Gunshot #6 sound
// id: 47
sound		weaponfx00_shot6_snd = <shot3.wav>;
// Entry: Gunshot #6 volume
// id: 48
define	weaponfx00_shot6_vol = 1275;


// Entry: Explosion Sound
// id: 51
sound 	weaponfx00_explo_snd = <explosion.wav>;


// Desc: play the empty weapon sound
// use->	ent_playsound(ent,weaponfx00_empty_snd,weaponfx00_empty_vol);

// Desc: play the reload weapon sound
// use->	ent_playsound(ent,weaponfx00_reload_snd,weaponfx00_reload_vol);

// Desc: play weapon firing noise
function WeaponFX00_Play_Shot(ent,id)
{
	if(id<1)	// 0
	{
		return;
	}
	if(id<2) // 1
	{
		ent_playsound(ent,weaponfx00_shot1_snd,weaponfx00_shot1_vol);
		return;
	}
	if(id<3) // 2
	{
		ent_playsound(ent,weaponfx00_shot2_snd,weaponfx00_shot2_vol);
		return;
	}
	if(id<4) // 3
	{
		ent_playsound(ent,weaponfx00_shot3_snd,weaponfx00_shot3_vol);
		return;
	}
	if(id<5) // 4
	{
		ent_playsound(ent,weaponfx00_shot4_snd,weaponfx00_shot4_vol);
		return;
	}
	if(id<6) // 5
	{
		ent_playsound(ent,weaponfx00_shot5_snd,weaponfx00_shot5_vol);
		return;
	}
	// 6
	ent_playsound(ent,weaponfx00_shot6_snd,weaponfx00_shot6_vol);

}


// DESC: Draws an animated explosion model.
// Note: Set temp equal to the radius of the explosion before calling
function WeaponFX00_Anim_Explode()
{
	var	my_range;
	var	my_scale_max;
	var	my_scale_inc;
	var 	my_snd_handle;

	my_range = temp.x;
	my_scale_max = temp/15; // you._shell_radius__030/15;    // explosion model has a width of 15 quants
	my_scale_inc = my_scale_max/2;
  	MY.PASSABLE = ON;	// don't push the player through walls


	if(temp.y == 1)
	{
		MY.TRANSPARENT = ON;
		MY.ALPHA = 45;
			MY.LIGHTRED = 255;
		MY.LIGHTGREEN = 128;
		MY.LIGHTBLUE = 128;
		MY.AMBIENT = 100;
		MY.LIGHTRANGE = 50;

		MY.SCALE_X = 1;
	}
	else
	{
		MY.invisible = on;
	}

	wait(1);

	// play sound
	// must wait one turn before we can play the sound
   my_snd_handle = ent_playsound(ME,weaponfx00_explo_snd,my_range*2);      // note: sound range is 10x volume

	if(MY.invisible == OFF)   // invisible is on if we didn't want to show a flash animation
	{
		// scale and rotate
		while(MY.SCALE_X < my_scale_max)
		{
			MY.LIGHTRANGE = 64 * MY.SCALE_X;
			MY.PAN += 15*TIME;
			MY.TILT += 5*TIME;
			temp = (my_scale_inc * TIME);
			MY.SCALE_X += temp;
			MY.SCALE_Y += temp;
			MY.SCALE_Z += temp;
 			wait(1);
		}

		// hide effect and wait for sound to finish
		my.invisible = ON;
		MY.LIGHTRANGE = 0;
	}


	while(snd_playing(my_snd_handle) == 1) { sleep(1); }

	ent_remove(ME);
}




endif; // weaponfx00_wdl